package com.alonginfo.framework.config;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.spring.boot.autoconfigure.DruidDataSourceBuilder;
import com.alibaba.druid.spring.boot.autoconfigure.properties.DruidStatProperties;
import com.alibaba.druid.util.Utils;
import com.alonginfo.common.utils.spring.SpringUtils;
import com.alonginfo.framework.aspectj.lang.enums.DataSourceType;
import com.alonginfo.framework.config.properties.DruidProperties;
import com.alonginfo.framework.datasource.DynamicDataSource;
import com.alonginfo.project.business.domain.MxDatabase;
import com.alonginfo.project.business.service.IMxDatabaseService;
import com.alonginfo.project.tool.Sm4Utils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import javax.annotation.PostConstruct;
import javax.servlet.*;
import javax.sql.DataSource;
import java.io.IOException;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * druid 配置多数据源
 */
@Configuration
public class DruidConfig {

    @Autowired
    IMxDatabaseService service;


    @Bean
//    @ConfigurationProperties("spring.datasource.druid.master")
    public DataSource masterDataSource(DruidProperties druidProperties) throws Exception {
        DruidDataSource dataSource = DruidDataSourceBuilder.create().build();
        DruidDataSource druidDataSource = druidProperties.dataSource(dataSource);
        return druidDataSource;
    }

    @Bean
    @ConfigurationProperties("spring.datasource.druid.slave")
    @ConditionalOnProperty(prefix = "spring.datasource.druid.slave", name = "enabled", havingValue = "true")
    public DataSource slaveDataSource(DruidProperties druidProperties) throws Exception {
        DruidDataSource dataSource = DruidDataSourceBuilder.create().build();
        return druidProperties.dataSource(dataSource);
    }

    @Bean(name = "dynamicDataSource")
    @Primary
    public DynamicDataSource dataSource(DataSource masterDataSource) {
        Map<Object, Object> targetDataSources = new HashMap<>();
        targetDataSources.put(DataSourceType.MASTER.name(), masterDataSource);
        setDataSource(targetDataSources, DataSourceType.SLAVE.name(), "slaveDataSource");
        return new DynamicDataSource(masterDataSource, targetDataSources);
    }
    
    /**
     * 设置数据源
     * 
     * @param targetDataSources 备选数据源集合
     * @param sourceName 数据源名称
     * @param beanName bean名称
     */
    public void setDataSource(Map<Object, Object> targetDataSources, String sourceName, String beanName) {
        try {
            DataSource dataSource = SpringUtils.getBean(beanName);
            targetDataSources.put(sourceName, dataSource);
        } catch (Exception e) {
        }
    }

    /**
     * 去除监控页面底部的广告
     */
    @SuppressWarnings({ "rawtypes", "unchecked" })
//    @Bean
    @ConditionalOnProperty(name = "spring.datasource.druid.statViewServlet.enabled", havingValue = "false")
    public FilterRegistrationBean removeDruidFilterRegistrationBean(DruidStatProperties properties) {
        // 获取web监控页面的参数
        DruidStatProperties.StatViewServlet config = properties.getStatViewServlet();
        // 提取common.js的配置路径
        String pattern = config.getUrlPattern() != null ? config.getUrlPattern() : "/druid/*";
        String commonJsPattern = pattern.replaceAll("\\*", "js/common.js");
        final String filePath = "support/http/resources/js/common.js";
        // 创建filter进行过滤
        Filter filter = new Filter() {
            @Override
            public void init(javax.servlet.FilterConfig filterConfig) throws ServletException {
            }
            @Override
            public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
                    throws IOException, ServletException {
                chain.doFilter(request, response);
                // 重置缓冲区，响应头不会被重置
                response.resetBuffer();
                // 获取common.js
                String text = Utils.readFromResource(filePath);
                // 正则替换banner, 除去底部的广告信息
                text = text.replaceAll("<a.*?banner\"></a><br/>", "");
                text = text.replaceAll("powered.*?shrek.wang</a>", "");
                response.getWriter().write(text);
            }
            @Override
            public void destroy() {
            }
        };
        FilterRegistrationBean registrationBean = new FilterRegistrationBean();
        registrationBean.setFilter(filter);
        registrationBean.addUrlPatterns(commonJsPattern);
        return registrationBean;
    }


    /**
     * 修改数据源内容  多添加几个业务数据源
     */
    @PostConstruct
    public void init(){
        List<MxDatabase> mxDatabases = service.selectMxDatabaseList(null);
        // 获取动态数据源
        DynamicDataSource dynamicDataSource = SpringUtils.getBean("dynamicDataSource");

        // 拿到主数据源   从数据源
        Map<Object, Object> targetDataSources = new HashMap<>();
        setDataSource(targetDataSources, DataSourceType.MASTER.name(), "masterDataSource");
        setDataSource(targetDataSources, DataSourceType.SLAVE.name(), "slaveDataSource");
        // 拿到业务数据源
        mxDatabases.stream().forEach(obj->{
            if ("Y".equals(obj.getIsOpen())) {
                String id = obj.getId();
                String url = obj.getUrl();
                String username = obj.getUsername();
                String password = obj.getPassword();
                String driverClass = obj.getDriverClass();
                // 初始化数据源
                DruidDataSource druidDataSource = new DruidDataSource();
                druidDataSource.setDriverClassName(driverClass);
                druidDataSource.setUrl(url);
                druidDataSource.setUsername(username);
                druidDataSource.setPassword(password);
                druidDataSource.setInitialSize(10); //初始化时建立物理连接的个数。初始化发生在显示调用init方法，或者第一次getConnection时
                druidDataSource.setMaxActive(100); //最大连接池数量
                druidDataSource.setMaxWait(60000); //获取连接时最大等待时间，单位毫秒。当链接数已经达到了最大链接数的时候，应用如果还要获取链接就会出现等待的现象，等待链接释放并回到链接池，如果等待的时间过长就应该踢掉这个等待，不然应用很可能出现雪崩现象
                druidDataSource.setMinIdle(40); //

                // 设置加密
                try {
                    druidDataSource.setFilters("config");
                    Properties p = new Properties();
                    p.setProperty("config.decrypt","true");
                    druidDataSource.setConnectProperties(p);
                } catch (SQLException e) {
                    e.printStackTrace();
                }
                // 添加到数据源集合
                targetDataSources.put(id,druidDataSource);
                dynamicDataSource.setTargetDataSources(targetDataSources);
                dynamicDataSource.afterPropertiesSet();
            }

        });
    }
}
