package com.alonginfo.project.business.controller;

import com.alibaba.fastjson.JSONObject;
import com.alonginfo.common.exception.CustomException;
import com.alonginfo.common.utils.DateUtils;
import com.alonginfo.common.utils.FileTypeDetector;
import com.alonginfo.common.utils.StringUtils;
import com.alonginfo.framework.config.MxConfig;
import com.alonginfo.framework.web.controller.BaseController;
import com.alonginfo.framework.web.domain.AjaxResult;
import com.alonginfo.framework.web.page.TableDataInfo;
import com.alonginfo.project.business.domain.MxTableColumn;
import com.alonginfo.project.business.service.IMxBusinessDataService;
import com.alonginfo.project.tool.FileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.text.DecimalFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

import static com.alonginfo.common.utils.DateUtils.YYYY_MM_DD_HH_MM_SS;

/**
 * @Description 业务数据管理
 * @Author Yalon
 * @Date 2020-03-13 12:36
 * @Version mxdata_v
 */
@Api("业务数据管理")
@RestController
@Slf4j
@RequestMapping("/businiss/mxbusinessdata")
public class MxBusinessDataController extends BaseController {

    /**
     * 业务数据管理业务层接口
     */
    @Autowired
    private IMxBusinessDataService mxBusinessDataService;

    /**
     * 查询业务表信息 表名称和表备注
     * tableName tableComment
     *
     * @return
     */
    @ApiOperation("查询业务表信息")
    @PostMapping("/getTableInfos")
    public AjaxResult getTableInfos() {
        try {
            return AjaxResult.success(mxBusinessDataService.getTableInfos());
        }catch (Exception e){
            e.printStackTrace();
            return AjaxResult.error("操作失败");
        }
    }

    /**
     * 根据表名称查询表字段信息
     *
     * @param tableName 表名称
     * @return
     */
    @ApiOperation("根据表名称查询表字段信息")
    @PostMapping("/getColumnByTableName")
    public AjaxResult getColumnByTableName(String tableName) {
        try {
            return AjaxResult.success(mxBusinessDataService.getColumnByTableName(tableName));
        }catch (Exception e){
            e.printStackTrace();
            return AjaxResult.error("操作失败");
        }
    }

    /**
     * 根据表名称查询表数据
     *
     * @param tableName 表名
     * @return
     */
    @ApiOperation("根据表名称查询表数据")
    @PostMapping("/getTableDataByTableName")
    public TableDataInfo getTableDataByTableName(String tableName) {
        startPage();
        try {
            return getDataTable(mxBusinessDataService.getTableDataByTableName(tableName));
        } catch (Exception e) {
            return new TableDataInfo();
        }
    }

    /**
     * 根据表名 表主键 主键值删除表数据
     *
     * @param tableName 表名
     * @param tableKey  表主键
     * @param keyValue  主键值
     * @return
     */
    @ApiOperation("根据表名 表主键 主键值删除表数据")
    @PostMapping("/removeDataByKey")
    public AjaxResult removeDataByKey(String tableName, String tableKey, String keyValue) {
        //检查必要参数是否为空
        if (StringUtils.isEmpty(new String[]{tableName, tableKey, keyValue})) {
            return AjaxResult.error("删除表数据必要参数有空！");
        }
        //组装参数map
        Map<String, Object> params = new HashMap<>(3);
        params.put("tableName", tableName);
        params.put("tableKey", tableKey);
        params.put("keyValue", keyValue);
        return toAjax(mxBusinessDataService.removeDataByKey(params));
    }

    /**
     * 根据表名 主键名 主键值  数据MAP 更新数据
     *
     * @param tableName 表名
     * @param tableKey  表主键
     * @param keyValue  主键值
     * @param dataJson  数据JSON
     * @return
     */
    @ApiOperation("根据表名 表主键 主键值   数据map 更新表数据")
    @PostMapping("/updateDataByKey")
    public AjaxResult updateDataByKey(String tableName, String tableKey, String keyValue, String dataJson) {
        //检查必要参数是否为空
        if (StringUtils.isEmpty(new String[]{tableName, tableKey, keyValue, dataJson})) {
            return AjaxResult.error("更新表数据必要参数有空！");
        }
        //组装数据map
        Map<String, Object> dataMap = JSONObject.parseObject(dataJson);
        //数据map为空 返回操作失败
        if (StringUtils.isEmpty(dataMap)) {
            return toAjax(0);
        }
        //组装参数map
        Map<String, Object> params = new HashMap<>(4);
        params.put("tableName", tableName);
        params.put("tableKey", tableKey);
        params.put("keyValue", keyValue);
        params.put("dataMap", dataMap);
        try {
            if (mxBusinessDataService.updateDataByKey(params) > 0) {
                return AjaxResult.success("修改数据成功");
            } else {
                return AjaxResult.error("修改0条数据");
            }
        } catch (Exception e) {
            log.error("修改业务表数据失败！", e);
            return AjaxResult.error("修改业务表数据失败！");
        }
    }

    /**
     * 根据表名 数据MAP 新增数据
     *
     * @param tableName 表名
     * @param dataJson  数据JSON
     * @return
     */
    @ApiOperation("根据表名 表主键 主键值   数据map 新增表数据")
    @PostMapping("/insertData")
    public AjaxResult insertData(String tableName, String dataJson) {
        //检查必要参数是否为空
        if (StringUtils.isEmpty(new String[]{tableName, dataJson})) {
            return AjaxResult.error("更新表数据必要参数有空！");
        }
        //组装数据map
        Map<String, Object> dataMap = JSONObject.parseObject(dataJson);
        //数据map为空 返回操作失败
        if (StringUtils.isEmpty(dataMap)) {
            return toAjax(0);
        }
        //组装参数map
        Map<String, Object> params = new HashMap<>(2);
        params.put("tableName", tableName);
        params.put("dataMap", dataMap);
        try {
            if (mxBusinessDataService.insertData(params) > 0) {
                return AjaxResult.success("新增数据成功");
            } else {
                return AjaxResult.error("新增0条数据");
            }
        } catch (Exception e) {
            log.error("插入业务表数据失败！", e);
            return AjaxResult.error("插入业务表数据失败！");
        }
    }

    /**
     * 导出表模板
     *
     * @param tableName 表名字
     * @return 下载url
     */
    @ApiOperation("导出表模板")
    @GetMapping("/exportHeadExcel")
    public AjaxResult exportHeadExcel(String tableName) {
        //获取列名
        List<MxTableColumn> columns = mxBusinessDataService.getColumnByTableName(tableName);
        //根据表名获取表注释名
        String tableNameCN = mxBusinessDataService.getTableCNName(tableName);
        //组装头map
        Map<String, Object> headMap = new LinkedHashMap<>();
        for (MxTableColumn column : columns) {
            if (!"PRI".equals(column.getColumnKey())
                    && !"tdate".equals(column.getColumnName())
                    && !"update_time".equals(column.getColumnName())) {
                headMap.put(column.getColumnName(), column.getColumnComment());
            }
        }
        //String fileName = exportExcel(headMap, new ArrayList<>(0), tableName);
        //return AjaxResult.success(fileName);
        String file = exportExcelTemplate(headMap, tableName, tableNameCN);
        return AjaxResult.success(file);
    }

    @ApiOperation("批量导入")
    @PostMapping("/importData")
    @Transactional(rollbackFor = RuntimeException.class)
    public AjaxResult importData(MultipartFile file, String tableName) throws IOException {
        String fileName2 = file.getOriginalFilename();
        String extension = "";
        boolean validExtension = false;
        boolean validExtension2 = false;
        if (fileName2 != null && fileName2.contains(".")) {
            extension = fileName2.substring(fileName2.lastIndexOf("."));
            String[] validExtensions = {"xlsx","xls"};
            String fileType = FileTypeDetector.getFileType(file);
            validExtension = FileUtil.isValidExtension(extension, validExtensions);
            validExtension2 = FileUtil.isValidExtension(fileType, validExtensions);
        }
        if(validExtension && validExtension2){
            List<Map<String, Object>> maps = null;
            try {
                maps = importExcel(file.getInputStream());
            } catch (Exception e) {
                log.error("导入数据时读取文件流失败", e);
                return toAjax(0);
            }
            if (StringUtils.isEmpty(maps)) {
                return AjaxResult.error("Excel文件无数据!");
            }
            mxBusinessDataService.delData(tableName);
            Integer rows = 0;
            for (Map<String, Object> dataMap : maps) {
                //组装参数map
                Map<String, Object> params = new HashMap<>(2);
                params.put("tableName", tableName);
                params.put("dataMap", dataMap);
                rows += mxBusinessDataService.insertData(params);
            }
            return toAjax(rows);
        }else{
            return AjaxResult.error();
        }
    }

    /**
     * 导出EXCEL
     *
     * @param header   excel头 {headCode:headName}
     * @param data     [{headCode:value}]
     * @param fileName 文件名
     * @return 文件名
     */
    private String exportExcel(Map<String, Object> header, List<Map<String, Object>> data, String fileName) {
        OutputStream out = null;
        Workbook wb = null;
        try {
            // 1.创建workbook
            wb = new SXSSFWorkbook(500);
            // 2.创建工作簿sheet
            Sheet sheet = wb.createSheet();
            // 3.设置两行表头
            Row row1 = sheet.createRow(0);
            Row row2 = sheet.createRow(1);
            //循环表头 插入工作簿的前两行
            int mapIndex = 0;
            for (Map.Entry<String, Object> entry : header.entrySet()) {
                //第一行写code
                Cell row1Cell = row1.createCell(mapIndex);
                row1Cell.setCellValue(entry.getKey());
                //第二行写标题
                Cell row2Cell = row2.createCell(mapIndex);
                row2Cell.setCellValue(entry.getValue().toString());
                //index+1
                mapIndex++;
            }
            // 隐藏第一行code
            row1.setZeroHeight(true);
            // 4.循环写入数据
            for (int i = 0; i < data.size(); i++) {
                Row row = sheet.createRow(i + 2);
                int j = 0;
                for (Map.Entry<String, Object> entry : header.entrySet()) {
                    Cell cell = row1.createCell(j);
                    cell.setCellValue(data.get(i).get(entry.getKey()).toString());
                    j++;
                }
            }
            // 5.处理文件名
            fileName = UUID.randomUUID().toString() + "_" + fileName + ".xlsx";
            String downloadPath = MxConfig.getDownloadPath() + fileName;
            File desc = new File(downloadPath);
            if (!desc.getParentFile().exists()) {
                desc.getParentFile().mkdirs();
            }
            out = new FileOutputStream(downloadPath);
            wb.write(out);
        } catch (Exception e) {
            log.error("导出Excel异常", e);
            throw new CustomException("导出Excel失败，请联系网站管理员！");
        } finally {
            if (wb != null) {
                try {
                    wb.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
        return fileName;
    }

    /**
     * 读取excel数据
     *
     * @param is 文件流
     * @return 数据集
     */
    private List<Map<String, Object>> importExcel(InputStream is) throws Exception {
        Workbook wb = null;
        // 1.获取workbook
        wb = WorkbookFactory.create(is);
        // 2.获取第一个工作簿
        Sheet sheet = wb.getSheetAt(0);
        if (sheet == null) {
            throw new IOException("文件sheet不存在");
        }
        // 3.获取表头 构造map
        Map<Integer, String> headCodeMap = new HashMap<>();
        Row headCode = sheet.getRow(0);
        for (int i = 0; i < headCode.getPhysicalNumberOfCells(); i++) {
            Cell cell = headCode.getCell(i);
            String value = null;
            if (cell != null) {
                value = cell.getStringCellValue();
            }
            headCodeMap.put(i, value);
        }
        //循环数据 构造数据list
        int rows = sheet.getPhysicalNumberOfRows();
        List<Map<String, Object>> dataList = new ArrayList<>();
        for (int i = 3; i < rows; i++) {
            Map<String, Object> dataMap = new HashMap<>();
            Row row = sheet.getRow(i);
            for (Map.Entry<Integer, String> entry : headCodeMap.entrySet()) {
                Cell cell = row.getCell(entry.getKey());
                Object val = null;
                if (cell != null) {
                    if (cell.getCellTypeEnum() == CellType.NUMERIC || cell.getCellTypeEnum() == CellType.FORMULA) {
                        val = cell.getNumericCellValue();
                        if (HSSFDateUtil.isCellDateFormatted(cell)) {
                            // POI Excel 日期格式转换
                            val = DateUtils.parseDateToStr(YYYY_MM_DD_HH_MM_SS, DateUtil.getJavaDate((Double) val));
                        } else {
                            if ((Double) val % 1 > 0) {
                                val = new DecimalFormat("0.00").format(val);
                            } else {
                                val = new DecimalFormat("0").format(val);
                            }
                        }
                    } else if (cell.getCellTypeEnum() == CellType.STRING) {
                        val = cell.getStringCellValue();
                    } else if (cell.getCellTypeEnum() == CellType.BOOLEAN) {
                        val = cell.getBooleanCellValue();
                    } else if (cell.getCellTypeEnum() == CellType.ERROR) {
                        val = cell.getErrorCellValue();
                    }

                }
                if (val != null) {
                    dataMap.put(entry.getValue(), val.toString());
                }
            }
            //避免空行情况 一直添加空map进去
            if (!CollectionUtils.isEmpty(dataMap)) {
                dataList.add(dataMap);
            }
        }
        return dataList;
    }

    /**
     * 业务数据表 excel模板 (新)
     *
     * @param header      表头字段集合
     * @param tableName   表名
     * @param tableCNName 表中文注释名称
     */
    private String exportExcelTemplate(Map<String, Object> header, String tableName, String tableCNName) {
        OutputStream out = null;
        Workbook workbook = null;
        try {
            //创建工作簿
            workbook = new XSSFWorkbook();
            //创建sheet
            Sheet sheet = workbook.createSheet();

            //公共样式
            CellStyle commonStyle = commonExcelStyle(workbook);

            //定义样式
            //表名行样式定义
            CellStyle tableNameCellStyle = workbook.createCellStyle();
            tableNameCellStyle.cloneStyleFrom(commonStyle);
            //设置背景色
            tableNameCellStyle.setFillForegroundColor(IndexedColors.GREY_50_PERCENT.getIndex());
            tableNameCellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);

            //创建字体
            Font font = workbook.createFont();
            font.setFontName("Arial");
            font.setBold(true);
            font.setColor(IndexedColors.WHITE.getIndex());
            font.setFontHeightInPoints((short) 18);
            tableNameCellStyle.setFont(font);


            //字段中文名行样式定义
            CellStyle columnNameCellStyle = workbook.createCellStyle();
            columnNameCellStyle.cloneStyleFrom(commonStyle);
            Font secondFont = workbook.createFont();
            secondFont.setFontName("Arial");
            secondFont.setFontHeightInPoints((short) 12);
            secondFont.setBold(true);
            columnNameCellStyle.setFont(secondFont);

            //创建三行
            //第一行存放字段名  第二行表中文注释  第三行字段中文名
            Row row0 = sheet.createRow(0);
            Row row1 = sheet.createRow(1);
            Row row2 = sheet.createRow(2);

            int cellIndex = 0;
            for (Map.Entry<String, Object> entry : header.entrySet()) {
                String key = entry.getKey();
                Object value = entry.getValue();
                //第一行存放字段名
                Cell cell0 = row0.createCell(cellIndex);
                cell0.setCellValue(key);
                //第二行表中文注释  单元格合并
                Cell cell1 = row1.createCell(cellIndex);
                cell1.setCellStyle(tableNameCellStyle);
                //第三行字段中文名
                Cell cell2 = row2.createCell(cellIndex);
                cell2.setCellValue(value.toString());
                cell2.setCellStyle(columnNameCellStyle);
                cellIndex++;
            }
            for (int i = 3; i <= 50; i++) {
                Row row = sheet.createRow(i);
                row.setHeightInPoints(25);
                for (int j = 0; j < header.entrySet().size(); j++) {
                    Cell cell = row.createCell(j);
                    cell.setCellStyle(commonStyle);
                }
            }
            //第一行隐藏
            row0.setZeroHeight(true);
            row1.setHeightInPoints(35);
            row2.setHeightInPoints(30);
            //第二行合并
            sheet.addMergedRegion(new CellRangeAddress(1, 1, 0, cellIndex - 1));
            row1.getCell(0).setCellValue(tableCNName);

            //列宽
            for (int i = 0; i <= cellIndex - 1; i++) {
                sheet.setColumnWidth(i, 30 * 256);
            }

            // 5.处理文件名
            //tableName = UUID.randomUUID().toString() + "_" + tableName + ".xlsx";
            tableName = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + "_" + tableCNName + ".xlsx";
            String downloadPath = MxConfig.getDownloadPath() + tableName;
            File desc = new File(downloadPath);
            if (!desc.getParentFile().exists()) {
                desc.getParentFile().mkdirs();
            }
            out = new FileOutputStream(downloadPath);
            workbook.write(out);
        } catch (Exception e) {
            log.error("导出Excel异常", e);
            throw new CustomException("导出Excel失败，请联系网站管理员！");
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
        return tableName;
    }

    /**
     * Excel公共样式
     *
     * @param workbook
     * @return
     */
    private CellStyle commonExcelStyle(Workbook workbook) {
        CellStyle cellStyle = workbook.createCellStyle();
        //设置边框 BorderStyle.THIN 细线
        cellStyle.setBorderTop(BorderStyle.THIN);
        cellStyle.setBorderBottom(BorderStyle.THIN);
        cellStyle.setBorderLeft(BorderStyle.THIN);
        cellStyle.setBorderRight(BorderStyle.THIN);
        //水平居中、垂直居中
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        return cellStyle;
    }
}
