package com.alonginfo.project.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alonginfo.common.utils.IdUtils;
import com.alonginfo.common.utils.StringUtils;
import com.alonginfo.framework.datasource.DynamicDataSourceContextHolder;
import com.alonginfo.project.business.domain.DataSetMenu;
import com.alonginfo.project.business.domain.MxDataGroup;
import com.alonginfo.project.business.domain.MxDataSet;
import com.alonginfo.project.business.domain.MxDataSetKey;
import com.alonginfo.project.business.mapper.MxDataSetMapper;
import com.alonginfo.project.business.service.IMxDataSetService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author Li先生
 * 数据集管理
 *
 */
@Slf4j
@Service
public class MxDataSetServiceImpl implements IMxDataSetService {

    @Autowired
    MxDataSetMapper mapper;

    /**
     * 获取数据集分组列表
     * @return
     */
    @Override
    public List<DataSetMenu> selectMxDataGroupList() {
        List<DataSetMenu> res = new ArrayList<>();
        List<MxDataGroup> mxDataGroups =  mapper.selectMxDataGroupList();
        mxDataGroups.stream().forEach((dataGtoup->{
            DataSetMenu dataSetMenu = new DataSetMenu();
            dataSetMenu.setId(dataGtoup.getDGroupId());
            dataSetMenu.setName(dataGtoup.getDGroupName());
            dataSetMenu.setDGroupSort(dataGtoup.getDGroupSort());
            dataSetMenu.setChildren(getDataSetListByDGroupId(dataGtoup.getDGroupId()));
            res.add(dataSetMenu);
        }));

        return res;
    }

    /**
     * 添加业务分类
     * @param mxDataGroup
     */
    @Override
    public int addMxDataGroup(MxDataGroup mxDataGroup) {
        mxDataGroup.setDGroupId(IdUtils.fastUUID());
        return  mapper.addMxDataGroup(mxDataGroup);
    }

    /**
     * 根据数据集分组id获取数据集
     * @param dGroupId
     * @return
     */
    @Override
    public List<DataSetMenu> getDataSetListByDGroupId(String dGroupId) {
         List<DataSetMenu> res = new ArrayList<>();
        List<MxDataSet> mxDataSets = mapper.getDataSetListByDGroupId(dGroupId);
        mxDataSets.stream().forEach(mxDataSet -> {
            DataSetMenu dataSetMenu = new DataSetMenu();
            dataSetMenu.setName(mxDataSet.getDsName());
            dataSetMenu.setId(mxDataSet.getDsId());
            res.add(dataSetMenu);
        });
        return res;
    }





    /**
     * 根据数据集 （数据源id+sql） 展示查询结果
     * @param mxDataSet
     * @return
     */
    @Override
    public List<Map<String, Object>> executeSql(MxDataSet mxDataSet) {
        Map<String, Object> params = new HashMap<>();
        try {
            params = JSONObject.parseObject(Optional.ofNullable(mxDataSet.getParam()).orElse("{}"), Map.class);
        } catch (Exception e) {
            log.info("数据集参数格式有误！");
        }

        params.put("dataSql", mxDataSet.getDataSql());
        List<Map<String, Object>> res = mapper.executeSql(params);
        return res;
    }

    /**
     * 添加数据集
     * @param mxDataSet
     * @return
     */
    @Override
    public int addMxDataSet(MxDataSet mxDataSet) {
        String dsId = "";
        // 修改  先删除再新增
        if (StringUtils.isNotBlank(mxDataSet.getDsId())) {
            dsId = mxDataSet.getDsId();
            deleteMxDataSet(dsId);
        } else {
            // 新增
            dsId = IdUtils.fastUUID();
            mxDataSet.setDsId(dsId);
        }

        // 切换数据源
        // change数据源
        DynamicDataSourceContextHolder.setDataSourceType(mxDataSet.getDatabaseId());

        Map<String, Object> params = new HashMap<>();
        try {
            params = JSONObject.parseObject(Optional.ofNullable(mxDataSet.getParam()).orElse("{}"), Map.class);
        } catch (Exception e) {
            log.info("数据集参数格式有误！");
        }
        params.put("dataSql", mxDataSet.getDataSql());
        List<Map<String,Object>> data = mapper.executeSql(params);

        // 切换到主数据源
        DynamicDataSourceContextHolder.setDataSourceType("master");

        if (data != null && data.size() >0) {
            Map<String,Object> tmpData = data.get(0);
            tmpData.keySet().stream().forEach(key->{
                MxDataSetKey mxDataSetKey = new MxDataSetKey();
                // 关联数据集
                mxDataSetKey.setDsId(mxDataSet.getDsId());
                // key主键
                mxDataSetKey.setKeyId(IdUtils.fastUUID());
                // key名称
                mxDataSetKey.setKeyName(key);
                // 保存数据集key
                mapper.addMxDtaSetKey(mxDataSetKey);
            });
            // 添加数据集
            return  mapper.addDataSet(mxDataSet);
        }
        return  0;
    }

    /**
     * 删除数据集
     * @param dsId
     * @return
     */
    @Override
    public int deleteMxDataSet(String dsId) {
        // 先删除数据集相关的key
        mapper.deleteMxDataSetKeyByDsId(dsId);
        // 删除数据集
        return mapper.deleteMxDataSetById(dsId);
    }

    /**
     * 删除数据集分组
     * @param dGroupId
     * @return
     */
    @Override
    public int deleteMxDataGroup(String dGroupId) {
        // 先删除数据分组下所有数据集
        deleteMxDataSetByDGroupId(dGroupId);
        // 删除分组
        return mapper.deleteMxDataGroup(dGroupId);
    }

    /**
     *  删除数据集分组下的所有数据集
     * @param dGroupId
     */
    private void deleteMxDataSetByDGroupId(String dGroupId) {
        List<MxDataSet> dataSets = mapper.getDataSetListByDGroupId(dGroupId);
        dataSets.stream().forEach(item->{
            deleteMxDataSet(item.getDsId());
        });
    }

    /**
     * 修改数据集分组
     * @param mxDataGroup
     * @return
     */
    @Override
    public int editMxDataGroup(MxDataGroup mxDataGroup) {
        return mapper.editMxDataGroup(mxDataGroup);
    }

    /**
     * 根据数据集id 获取实例
     * @param dsId
     * @return
     */
    @Override
    public MxDataSet getMxDataSet(String dsId) {
        return mapper.getMxDataSet(dsId);
    }

    /**
     * 获取所有的mx_data_set_key
     * @return
     */
    @Override
    public List<MxDataSetKey> listMxDataSetKey() {
        return mapper.listMxDataSetKey();
    }
}
