package com.alonginfo.project.business.service.impl;

import com.alonginfo.common.enums.ScheduleConstants;
import com.alonginfo.common.exception.job.TaskException;
import com.alonginfo.project.business.mapper.MxQuartzMapper;
import com.alonginfo.project.business.service.IMxQuartzService;
import com.alonginfo.quartz.domain.SysJob;
import com.alonginfo.quartz.utils.ScheduleUtils;
import com.alonginfo.utils.SecurityUtils;
import org.quartz.JobDataMap;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * @Description  定时任务 业务层实现
 * @Author Haoxs
 * @Date 2021-04-26 16:20
 */
@Service
public class MxQuartzServiceImpl implements IMxQuartzService {

    @Autowired
    private MxQuartzMapper mxQuartzMapper;

    @Autowired
    private Scheduler scheduler;

    /**
     * 获取全部定时任务列表
     * @return
     */
    @Override
    public List<Map<String, Object>> getQuartzList(SysJob job) {
        List<Map<String, Object>> quartzList = mxQuartzMapper.getQuartzList(job);
        return quartzList;
    }

    /**
     * 通过id查询定时任务
     * @param jobId
     * @return
     */
    @Override
    public SysJob selectJobById(Long jobId) {
        return mxQuartzMapper.selectJobById(jobId);
    }

    /**
     * 修改调度任务状态
     * @param newJob
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int changeStatus(SysJob newJob) throws SchedulerException {
        int rows = 0;
        String status = newJob.getStatus();
        if (ScheduleConstants.Status.NORMAL.getValue().equals(status)) {
            rows = resumeJob(newJob);
        } else if (ScheduleConstants.Status.PAUSE.getValue().equals(status)) {
            rows = pause(newJob);
        }
        return rows;
    }

    /**
     * 恢复任务
     * @param job
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int resumeJob(SysJob job) throws SchedulerException {
        Long jobId = job.getJobId();
        String jobGroup = job.getJobGroup();
        job.setStatus(ScheduleConstants.Status.NORMAL.getValue());
        int rows = mxQuartzMapper.updateJob(job);
        if (rows > 0)
        {
            scheduler.resumeJob(ScheduleUtils.getJobKey(jobId, jobGroup));
        }
        return rows;
    }

    /**
     * 暂停任务
     * @param job
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int pause(SysJob job) throws SchedulerException {
        Long jobId = job.getJobId();
        String group = job.getJobGroup();
        job.setStatus(ScheduleConstants.Status.PAUSE.getValue());
        int rows = mxQuartzMapper.updateJob(job);
        if (rows > 0) {
            scheduler.pauseJob(ScheduleUtils.getJobKey(jobId, group));
        }
        return rows;
    }

    /**
     * 新增定时任务
     * @param job
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertJob(SysJob job) throws TaskException, SchedulerException {
        job.setCreateBy(SecurityUtils.getUsername());
        //计划执行错误策略
        job.setMisfirePolicy("3");
        job.setStatus(ScheduleConstants.Status.PAUSE.getValue());
        int rows = mxQuartzMapper.insertJob(job);
        if (rows > 0) {
            ScheduleUtils.createScheduleJob(scheduler, job);
        }
        return rows;
    }

    /**
     * 立即运行定时任务
     * @param job
     */
    @Override
    public void runJob(SysJob job) throws SchedulerException {
        Long jobId = job.getJobId();
        String jobGroup = job.getJobGroup();
        SysJob properties = selectJobById(job.getJobId());
        // 参数
        JobDataMap dataMap = new JobDataMap();
        dataMap.put(ScheduleConstants.TASK_PROPERTIES, properties);
        scheduler.triggerJob(ScheduleUtils.getJobKey(jobId, jobGroup), dataMap);
    }

    /**
     * 立即运行所有定时任务
     * @param jobList
     */
    @Override
    public void runAllJob(List<SysJob> jobList) throws SchedulerException {
        for (SysJob job : jobList) {
            Long jobId = job.getJobId();
            String jobGroup = job.getJobGroup();
            SysJob properties = selectJobById(job.getJobId());
            // 参数
            JobDataMap dataMap = new JobDataMap();
            dataMap.put(ScheduleConstants.TASK_PROPERTIES, properties);
            scheduler.triggerJob(ScheduleUtils.getJobKey(jobId, jobGroup), dataMap);
        }
    }
}
