package com.alonginfo.project.engine.service.impl;

import com.alonginfo.common.constant.DatavConstans;
import com.alonginfo.common.utils.StringUtils;
import com.alonginfo.framework.redis.RedisCache;
import com.alonginfo.project.engine.domain.MxPages;
import com.alonginfo.project.engine.domain.MxProject;
import com.alonginfo.project.engine.domain.ThemeManager;
import com.alonginfo.project.engine.mapper.MxPagesMapper;
import com.alonginfo.project.engine.mapper.MxProjectMapper;
import com.alonginfo.project.engine.mapper.ThemeManagerMapper;
import com.alonginfo.project.engine.service.IMxPagesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 主题页面管理 -- 服务实现层
 * @author shidj
 */
@Service
public class MxPagesServiceImpl implements IMxPagesService {

    @Autowired
    RedisCache redisCache;

    @Autowired
    private MxPagesMapper pagesMapper;

    @Autowired
    private ThemeManagerMapper themeManagerMapper;

    @Autowired
    private MxProjectMapper mxProjectMapper;

    /**
     * 获取工程页面信息
     * 根据工程id 或页面id查询
     * @param mxPages
     * @return
     */
    @Override
    public List<MxPages> getPagesForId(MxPages mxPages) {
        return pagesMapper.getPages(mxPages);
    }

    /**
     * 获取工程页面信息
     * 根据工程id 或页面id查询
     * @param mxPages
     * @return
     */
    @Override
    public List<MxPages> getPages(MxPages mxPages) {
        List<MxPages> listPages = redisCache.getCacheList("all_Pages");
        if (listPages == null || listPages.size() == 0) {
            listPages = pagesMapper.getPages(mxPages);
            redisCache.setCacheList("all_Pages",listPages);
        }
        return listPages;
    }

    /**
     * 判断页面名称是否存在
     * @param mxPages
     * @return
     */
    @Override
    public String checkPageNameUnique(MxPages mxPages) {
        String pageId = StringUtils.isNull(mxPages.getPageId()) ? "-1" : mxPages.getPageId();
        MxPages pages = pagesMapper.selectPageForName(mxPages);
        if (StringUtils.isNotNull(pages) && !pageId.equals(pages.getPageId())){
            return DatavConstans.NOT_UNIQUE;
        }
        return DatavConstans.UNIQUE;
    }

    /**
     * 插入主题页面信息
     * @param mxPages
     * @return
     */
    @Override
    public int insertPages(MxPages mxPages) {
        int row = pagesMapper.insertPages(mxPages);
        if (row > 0) {
            redisCache.deleteObject("all_Pages");
        }
        return row;
    }

    /**
     * 获取页面相关联信息
     * @param mxPages
     * @return
     */
    @Override
    public Map<String, Object> getReleaseData(MxPages mxPages) {
        // 准备入redis
        String  pageId = mxPages.getPageId();
        Map<String, Object> map = redisCache.getCacheMap(pageId);
        if (map != null && map.size() != 0) {
            return  map;
        }
        // 根据页面id获取页面信息
        MxPages pageInfo = pagesMapper.getPageInfo(mxPages);

        if("0".equals(pageInfo.getPagePublish())) {// 未发布
            map.put("notPublish","0");
            return map;
        }
        // 判断设计主题json
        if("1".equals(pageInfo.getIsOwnChartTheme()) && StringUtils.isNotEmpty(pageInfo.getThemeId())) {
            ThemeManager th = new ThemeManager();
            th.setThemeId(pageInfo.getThemeId());
            ThemeManager chartTheme = themeManagerMapper.getInfoChartThemeId(th);
            pageInfo.setJsonText(chartTheme.getJsonText());
        }
        map.put("pageMetadata", pageInfo);

        MxProject mx = new MxProject();
        mx.setProId(pageInfo.getProId());
        MxProject mxProject = mxProjectMapper.getInfo(mx);

        if (StringUtils.isNotEmpty(mxProject.getThemeId())) {
            ThemeManager th = new ThemeManager();
            th.setThemeId(mxProject.getThemeId());
            ThemeManager chartTheme = themeManagerMapper.getInfoChartThemeId(th);
            mxProject.setJsonText(chartTheme.getJsonText());
        }
        map.put("projectInfo", mxProject);
        // 放入redis
        redisCache.setCacheMap(pageId,map);
        return map;
    }

    /**
     * 更新页面信息
     * @param mxPages
     * @return
     */
    @Override
    public int updatePages(MxPages mxPages) {
        int rows = pagesMapper.updatePages(mxPages);
        if (rows > 0) {// 更新页面信息，清除缓存
            redisCache.deleteObject(mxPages.getPageId());
        }
        return rows;
    }

    /**
     * 主题发布
     * @param mxPages
     * @return
     */
    @Override
    public int releasePage(MxPages mxPages) {
        if (StringUtils.isNotEmpty(mxPages.getPageId())) {
            return pagesMapper.releasePage(mxPages);
        } else {
            return -1;
        }
    }

    /**
     * 删除主题
     * @param pageId
     * @return
     */
    @Override
    public int mxDeletePage(String pageId) {
        return pagesMapper.mxDeletePage(pageId);
    }
}
