package com.alonginfo.project.gansu.serviceimpl.marketsurvey;

import com.alonginfo.project.gansu.domain.*;
import com.alonginfo.project.gansu.mapper.MarketSurveyMapper;
import com.alonginfo.project.gansu.service.SgccAddressService;
import com.alonginfo.project.gansu.service.marketsurvey.IMarketSurveyService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * @author: HaoXs
 * @Date: 2023/2/7 15:08
 * @Description: 市场概况业务层
 */
@Service
@Slf4j
public class MarketSurveyServiceImpl implements IMarketSurveyService {

    @Autowired
    private MarketSurveyMapper marketSurveyMapper;

    /**
     * (同步数据)当前市场主体情况
     *
     * @return
     */
    @Override
    public SubjectInfoDTO getMarketSubjectInfo() {
        //市场主体占比
        SubjectRatioPo subjectRatioPo = marketSurveyMapper.getSubjectInfo();
        EnterprisePo enterprisePo = marketSurveyMapper.getEnterpriseInfo();
        SubjectRatioPo subjectInfo = ObjectUtils.isEmpty(subjectRatioPo) ? new SubjectRatioPo() : subjectRatioPo;
        EnterprisePo enterpriseInfo = ObjectUtils.isEmpty(enterprisePo) ? new EnterprisePo() : enterprisePo;

        SubjectInfoDTO subjectInfoDTO = new SubjectInfoDTO();
        BeanUtils.copyProperties(subjectInfo, subjectInfoDTO);
        BeanUtils.copyProperties(enterpriseInfo, subjectInfoDTO);
        return subjectInfoDTO;
    }

    /**
     * (导入数据)发电、用电
     *
     * @param flag 0-用电 1-发电
     * @return
     */
    @Override
    public List<String> getUseGeneration(String flag) {
        if (StringUtils.isEmpty(flag)) {
            return new ArrayList<>();
        }

        List<String> result = new ArrayList<>();
        if ("0".equals(flag)) {
            result = marketSurveyMapper.queryUseInfo();
        } else if ("1".equals(flag)) {
            result = marketSurveyMapper.queryGenerationInfo();
        } else {

        }
        return result;
    }

    /**
     * (导入数据)发电情况-各类型电量累计发用电
     *
     * @return
     */
    @Override
    public CountPowerByTypeDTO getCountPowerByType() {
        List<Map<String, String>> countPowers = marketSurveyMapper.queryCountPowerByType();
        if (StringUtils.isEmpty(countPowers)) {
            return new CountPowerByTypeDTO();
        }

        List<String> use = new LinkedList<>();
        List<String> generation = new LinkedList<>();
        for (Map<String, String> countPower : countPowers) {
            //发电/用电
            String dataType = Optional.ofNullable(countPower.get("dataType")).orElse("");
            String fire = countPower.get("fireElectric");
            String water = countPower.get("waterElectric");
            String wind = countPower.get("windElectric");
            String light = countPower.get("lightElectric");
            String other = countPower.get("other");

            if (dataType.contains("用电")) {
                use = Arrays.asList(fire, other, light, wind, water);
            } else if (dataType.contains("发电")) {
                generation = Arrays.asList(fire, other, light, wind, water);
            } else {

            }
        }
        return CountPowerByTypeDTO.builder()
                .electricityConsumption(use)
                .powerGeneration(generation)
                .build();
    }

    /**
     * (导入数据)发电情况-当年用电趋势
     *
     * @return
     */
    @Override
    public UseTrendCurrentYearDTO getUseTrendCurrentYear() {
        List<Map<String, String>> useTrends = marketSurveyMapper.queryUseTrendCurrentYear();
        if (StringUtils.isEmpty(useTrends)) {
            return new UseTrendCurrentYearDTO();
        }

        List<String> use = new LinkedList<>();
        List<String> generation = new LinkedList<>();
        for (int i = 1; i <= 12; i++) {
            Map<String, String> map = new HashMap<>();
            for (Map<String, String> useTrend : useTrends) {
                String dataMonth = useTrend.get("dataMonth");
                if (String.valueOf(i).equals(dataMonth)) {
                    map = useTrend;
                }
            }
            use.add(map.get("useElectric"));
            generation.add(map.get("generationElectric"));
        }
        return UseTrendCurrentYearDTO.builder()
                .electricityConsumption(use)
                .powerGeneration(generation)
                .build();
    }

    /**
     * (导入数据)当年交易量趋势
     * @return
     */
    @Override
    public TradeVolumeCurrentDTO getTradeVolumeTrendCurrent() {
        List<Map<String, String>> tradeVolumes = marketSurveyMapper.queryTradeVolumeTrendCurrent();
        if (StringUtils.isEmpty(tradeVolumes)) {
            return new TradeVolumeCurrentDTO();
        }

        List<String> innerPower = new LinkedList<>();
        List<String> outDeliveryPower = new LinkedList<>();
        List<String> outBuyPower = new LinkedList<>();
        List<String> innerPrice = new LinkedList<>();
        List<String> outDeliveryPrice = new LinkedList<>();
        List<String> outBuyPrice = new LinkedList<>();
        for (int i = 1; i <= 12; i++) {
            Map<String, String> item = new HashMap<>();
            for (Map<String, String> tradeVolume : tradeVolumes) {
                String dataMonth = tradeVolume.get("dataMonth");
                if (String.valueOf(i).equals(dataMonth)) {
                    item = tradeVolume;
                }
            }
            innerPower.add(item.get("provinceInnerPower"));
            outDeliveryPower.add(item.get("outDeliveryPower"));
            outBuyPower.add(item.get("outBuyPower"));
            innerPrice.add(item.get("provinceInnerPrice"));
            outDeliveryPrice.add(item.get("outDeliveryPrice"));
            outBuyPrice.add(item.get("outBuyPrice"));
        }
        return TradeVolumeCurrentDTO.builder()
                .electricityInProvince(innerPower)
                .powerToBeDeliveredToOutside(outDeliveryPower)
                .purchasedElectricity(outBuyPower)
                .inProvinceTheAverage(innerPrice)
                .averagePriceToBeDeliveredToOutside(outDeliveryPrice)
                .outsourcingAverage(outBuyPrice).build();
    }

    /**
     * (导入数据)当年市场化电量累计执行情况
     *
     * @return
     */
    @Override
    public MarketCountExecuteDTO getMarketCountExecute() {
        List<Map<String, String>> executes = marketSurveyMapper.queryMarketCountExecute();
        if (StringUtils.isEmpty(executes)) {
            return new MarketCountExecuteDTO();
        }
        //外送电量
        Map<String, String> outDelivery = executes.stream().filter(item -> "外送电量".equals(item.get("dataType"))).findFirst().orElse(new HashMap<>());
        //外购电量
        Map<String, String> outBuy = executes.stream().filter(item -> "外购电量".equals(item.get("dataType"))).findFirst().orElse(new HashMap<>());
        //省内直接交易
        Map<String, String> provinceDirect = executes.stream().filter(item -> "省内直接交易".equals(item.get("dataType"))).findFirst().orElse(new HashMap<>());
        //发电权交易
        Map<String, String> generation = executes.stream().filter(item -> "发电权交易".equals(item.get("dataType"))).findFirst().orElse(new HashMap<>());
        //现货交易
        Map<String, String> spots = executes.stream().filter(item -> "现货交易".equals(item.get("dataType"))).findFirst().orElse(new HashMap<>());

        return MarketCountExecuteDTO.builder()
                .powerToBeDeliveredToOutside(outDelivery)
                .purchasedElectricity(outBuy)
                .inProvinceTheOtc(provinceDirect)
                .powerGenerationRightTrad(generation)
                .spotTrad(spots).build();
    }

    /**
     * (导入数据)发电装机情况与利用小时数
     * @return
     */
    @Override
    public InstallAndHoursDTO getInstallAndHours() {
        Map<String, String> installAndHours = marketSurveyMapper.queryInstallAndHours();
        if (StringUtils.isEmpty(installAndHours)) {
            return new InstallAndHoursDTO();
        }
        //装机容量 【总装机容量，火，水，风，光】
        List<String> installedCapacityList = Arrays.asList(installAndHours.get("countInstallCapacity"), installAndHours.get("fireInstallCapacity"),
                                                       installAndHours.get("waterInstallCapacity"), installAndHours.get("windInstallCapacity"),
                                                       installAndHours.get("lightInstallCapacity"));
        //装机容量同比【总同比，火，水，风，光】
        List<String> installedCapacityComparedToSameList = Arrays.asList(installAndHours.get("countInstallYoy"), installAndHours.get("fireInstallYoy"),
                                                                     installAndHours.get("waterInstallYoy"), installAndHours.get("windInstallYoy"),
                                                                     installAndHours.get("lightInstallYoy"));
        //利用小时 【总利用小时，火，水，风，光】
        List<String> usingHourList = Arrays.asList(installAndHours.get("countUseHour"), installAndHours.get("fireUseHour"),
                                               installAndHours.get("waterUseHour"), installAndHours.get("windUseHour"),
                                               installAndHours.get("lightUseHour"));

        //利用小时同比 【总同比，火，水，风，光】
        List<String> usingHourComparedToSameList = Arrays.asList(installAndHours.get("countHourYoy"), installAndHours.get("fireHourYoy"),
                                                             installAndHours.get("waterHourYoy"), installAndHours.get("windHourYoy"),
                                                             installAndHours.get("lightHourYoy"));
        return InstallAndHoursDTO.builder()
                .installedCapacity(installedCapacityList)
                .installedCapacityComparedToSame(installedCapacityComparedToSameList)
                .usingHour(usingHourList)
                .usingHourComparedToSame(usingHourComparedToSameList).build();
    }


}
