package com.alonginfo.project.gansu.serviceimpl.provinceInner;

import com.alonginfo.project.gansu.domain.*;
import com.alonginfo.project.gansu.mapper.ProvinceInnerMapper;
import com.alonginfo.project.gansu.service.provinceinner.IProvinceInnerService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.Year;
import java.util.*;

/**
 * @author: HaoXs
 * @Date: 2023/5/18 16:14
 * @Description: 省内交易情况 业务层
 */
@Service
@Slf4j
public class ProvinceInnerServiceImpl implements IProvinceInnerService {

    @Autowired
    private ProvinceInnerMapper provinceInnerMapper;

    /**
     * 当年交易组织情况
     *
     * @return
     */
    @Override
    public PiDealOrganizeVo getDealOrganize() {
        //查询当年的数据
        PiDealOrganizeVo dealOrganize = provinceInnerMapper.getDealOrganize();
        return ObjectUtils.isEmpty(dealOrganize) ? new PiDealOrganizeVo() : dealOrganize;
    }

    /**
     * 当年市场主体参与情况
     *
     * @return
     */
    @Override
    public PiSubjectJoinVo getSubjectJoinInfo() {
        PiSubjectJoinVo subjectJoinInfo = provinceInnerMapper.getSubjectJoinInfo();
        return ObjectUtils.isEmpty(subjectJoinInfo) ? new PiSubjectJoinVo() : subjectJoinInfo;
    }

    /**
     * 各类清洁能源成交电量
     *
     * @return
     */
    @Override
    public List<PiCleanEnergyVo> getCleanEnergy() {
        List<PiCleanEnergyVo> cleanEnergy = provinceInnerMapper.getCleanEnergy();
        return CollectionUtils.isEmpty(cleanEnergy) ? new ArrayList<>() : cleanEnergy;
    }

    /**
     * 交易概览
     *
     * @return
     */
    @Override
    public PiDealSurveyDTO getDealSurvey() {
        //市场准入家数及同比 查询job_pi_subject_join 计算同比
        //当年市场准入家数
        String accessCurrentYear = provinceInnerMapper.getMarketAccessByYear(String.valueOf(Year.now().getValue()));
        //去年市场准入家数
        String accessLastYear = provinceInnerMapper.getMarketAccessByYear(String.valueOf(Year.now().getValue() - 1));
        //市场准入同比
        String accessRatio = "";
        if (StringUtils.isEmpty(accessLastYear)) {
            accessRatio = "100%";
            log.error("去年市场准入家数为0或NULL!");
        } else {
            accessRatio = new BigDecimal(accessCurrentYear).divide(new BigDecimal(accessLastYear), 2, BigDecimal.ROUND_HALF_DOWN) + "%";
        }
        //交易组织次数及同比、成交电量及同比
        List<Map<String, String>> surveyInfoLists = provinceInnerMapper.getSurveyInfo();
        //当年
        Map<String, String> currentMap = surveyInfoLists.stream().filter(f -> String.valueOf(Year.now().getValue()).equals(f.get("tradeYear"))).findFirst().orElse(new HashMap<>());
        //去年
        Map<String, String> lastMap = surveyInfoLists.stream().filter(f -> String.valueOf(Year.now().minusYears(1).getValue()).equals(f.get("tradeYear"))).findFirst().orElse(new HashMap<>());

        //交易组织次数、同比
        BigDecimal currentTradeNum = new BigDecimal(Optional.ofNullable(currentMap.get("tradeNum")).orElse("0"));
        BigDecimal lastTradeNum = new BigDecimal(Optional.ofNullable(lastMap.get("tradeNum")).orElse("0"));
        BigDecimal tradeYoy = lastTradeNum.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO : currentTradeNum.divide(lastTradeNum, 2, BigDecimal.ROUND_HALF_UP);
        //成交电量、同比
        BigDecimal currentVendeeEnergyNum = new BigDecimal(Optional.ofNullable(currentMap.get("vendeeEnergyNum")).orElse("0"));
        BigDecimal lastVendeeEnergyNum = new BigDecimal(Optional.ofNullable(lastMap.get("vendeeEnergyNum")).orElse("0"));
        BigDecimal vendeeEnergyYoy = lastVendeeEnergyNum.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO : currentVendeeEnergyNum.divide(lastVendeeEnergyNum, 2, BigDecimal.ROUND_HALF_UP);

        PiDealSurveyDTO piDealSurveyDTO = new PiDealSurveyDTO();
        piDealSurveyDTO.setAccessNum(accessCurrentYear);
        piDealSurveyDTO.setAccessYoy(accessRatio);
        piDealSurveyDTO.setTradeNum(currentTradeNum);
        piDealSurveyDTO.setTradeYoy(tradeYoy);
        piDealSurveyDTO.setVendeeEnergyNum(currentVendeeEnergyNum);
        piDealSurveyDTO.setVendeeEnergyYoy(vendeeEnergyYoy);
        return piDealSurveyDTO;
    }

    /**
     * 当年交易量价走势
     *
     * @return
     */
    @Override
    public PiPowerPriceTrendDTO getPowerPriceTrend() {
        List<PiPowerPriceTrendVo> powerPriceTrendList = provinceInnerMapper.getPowerPriceTrend();

        List<String> xAxis = new LinkedList<>();
        List<String> energy = new LinkedList<>();
        List<String> price = new LinkedList<>();
        powerPriceTrendList.forEach(piPowerPriceTrendVo -> {
            xAxis.add(piPowerPriceTrendVo.getTradeMonth());
            energy.add(piPowerPriceTrendVo.getVendeeEnergy());
            price.add(piPowerPriceTrendVo.getVendeePrice());
        });

        List<Map<String, Object>> seriesData = new ArrayList<Map<String, Object>>() {{
            add(new HashMap<String, Object>() {{
                put("name", "电量");
                put("data", energy);
            }});
            add(new HashMap<String, Object>() {{
                put("name", "电价");
                put("data", price);
            }});
        }};

        PiPowerPriceTrendDTO piPowerPriceTrendDTO = new PiPowerPriceTrendDTO();
        piPowerPriceTrendDTO.setXAxisData(xAxis);
        piPowerPriceTrendDTO.setSeriesData(seriesData);
        return piPowerPriceTrendDTO;
    }

    /**
     * （导入数据）地图弹框数据
     *
     * @return
     */
    @Override
    public MapTextDTO getMapText() {
        List<MapTextVO> mapTexts = provinceInnerMapper.queryMapText();
        if (com.alonginfo.common.utils.StringUtils.isEmpty(mapTexts)) {
            return new MapTextDTO();
        }
        MapTextDTO mapTextDTO = new MapTextDTO();
        for (MapTextVO mapText : mapTexts) {
            String cityName = mapText.getCityName();
            List<String> data = Arrays.asList(mapText.getTradeElectric());
            if ("酒泉市".contains(cityName)) {
                mapTextDTO.setJqs(data);
            } else if ("嘉峪关市".contains(cityName)) {
                mapTextDTO.setJygs(data);
            } else if ("张掖市".contains(cityName)) {
                mapTextDTO.setZys(data);
            } else if ("金昌市".contains(cityName)) {
                mapTextDTO.setJcs(data);
            } else if ("武威市".contains(cityName)) {
                mapTextDTO.setWws(data);
            } else if ("兰州市".contains(cityName)) {
                mapTextDTO.setLzs(data);
            } else if ("临夏回族自治州".contains(cityName)) {
                mapTextDTO.setLxhzzzq(data);
            } else if ("白银市".contains(cityName)) {
                mapTextDTO.setBys(data);
            } else if ("甘南藏族自治州".contains(cityName)) {
                mapTextDTO.setGnzzzzq(data);
            } else if ("定西市".contains(cityName)) {
                mapTextDTO.setDxs(data);
            } else if ("陇南市".contains(cityName)) {
                mapTextDTO.setLns(data);
            } else if ("天水市".contains(cityName)) {
                mapTextDTO.setTss(data);
            } else if ("平凉市".contains(cityName)) {
                mapTextDTO.setPls(data);
            } else if ("庆阳市".contains(cityName)) {
                mapTextDTO.setQys(data);
            } else {

            }
        }
        return mapTextDTO;
    }

    /**
     * （导入数据）历年降低购电成本与交易均价
     *
     * @return
     */
    @Override
    public CostAndAvgDTO getCostAndAvgByYear() {
        List<Map<String, String>> maps = provinceInnerMapper.queryCostAndAvgByYear();
        if (com.alonginfo.common.utils.StringUtils.isEmpty(maps)) {
            return new CostAndAvgDTO();
        }

        List<String> xData = new LinkedList<>();
        List<String> costData = new LinkedList<>();
        List<String> avgPriceData = new LinkedList<>();
        for (Map<String, String> map : maps) {
            xData.add(map.get("dataYear"));
            costData.add(map.get("costDown"));
            avgPriceData.add(map.get("avgPrice"));
        }
        CostAndAvgDTO costAndAvgDTO = new CostAndAvgDTO();
        costAndAvgDTO.setXData(xData);
        costAndAvgDTO.setCostDown(costData);
        costAndAvgDTO.setAvgPrice(avgPriceData);
        return costAndAvgDTO;
    }

    /**
     * （导入数据）历年交易规模趋势
     *
     * @return
     */
    @Override
    public TradeSizeDTO getTradeSizeByYear() {
        List<Map<String, String>> tradeSizeList = provinceInnerMapper.queryTradeSizeByYear();
        if (com.alonginfo.common.utils.StringUtils.isEmpty(tradeSizeList)) {
            return new TradeSizeDTO();
        }
        List<String> xData = new LinkedList<>();
        List<String> seriesData = new LinkedList<>();
        for (Map<String, String> map : tradeSizeList) {
            xData.add(map.get("dataYear"));
            seriesData.add(map.get("tradeValue"));
        }

        TradeSizeDTO tradeSizeDTO = new TradeSizeDTO();
        tradeSizeDTO.setXAxisData(xData);
        tradeSizeDTO.setSeriesData(seriesData);
        return tradeSizeDTO;
    }

}
