package com.alonginfo.project.gansu.sync.impl;

import com.alibaba.fastjson.JSON;
import com.alonginfo.common.utils.StringUtils;
import com.alonginfo.common.utils.http.HttpUtils;
import com.alonginfo.project.gansu.domain.*;
import com.alonginfo.project.gansu.mapper.*;
import com.alonginfo.project.gansu.service.SgccAddressService;
import com.alonginfo.project.gansu.sync.BilateralConsultDataSync;
import com.alonginfo.utils.DataPushUtils;
import com.lowagie.text.pdf.PdfReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

import static com.alonginfo.utils.DataPushUtils.handleSeqList;


/**
 * 甘肃二期-双边协商  动态数据获取
 *
 * @author HaoXs
 * @since 2022-03-14 11:49
 */
@Service
public class BilateralConsultDataSyncImpl implements BilateralConsultDataSync {

    private static final Logger logger = LoggerFactory.getLogger(BilateralConsultDataSyncImpl.class);

    @Value("${env.key}")
    private String ENV_KEY;

    @Value("${sgcc.publicKey}")
    private String PUBLIC_KEY;

    @Value("${notice.filePath}")
    private String NOTICE_FILE_PATH;

    @Value("${market.filePath}")
    private String MARKET_FILE_PATH;

    @Autowired
    private SgccAddressService sgccAddressService;

    @Autowired
    private TradeSeqDao tradeSeqDao;

    @Autowired
    private TradeNoticeDao tradeNoticeDao;

    @Autowired
    private TradeStatusDao tradeStatusDao;

    @Autowired
    private PriceTrendDao priceTrendDao;

    @Autowired
    private EnergyCensusDao energyCensusDao;

    @Autowired
    private PriceTrendReplayDao priceTrendReplayDao;

    @Autowired
    private RealTimeManyDayDao realTimeManyDayDao;

    @Autowired
    private UploadNoticeMapper uploadNoticeMapper;

    /**
     * 交互电力交易平台 == 获取最新交易序列信息
     *
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void getTradeSeqInfoList() {
        try {
            String sgccUrl = sgccAddressService.getSgccAddress(ENV_KEY);
            //调用最新交易序列信息接口
            String seqInfoResult = HttpUtils.sendPost(sgccUrl + "/dataPush/tradeSeq/selectTradeSeqInfoList", "", DataPushUtils.createToken(PUBLIC_KEY));
            logger.info("获取双边协商-最新交易序列信息, 接口调用结果: {}", seqInfoResult);
            if (seqInfoResult == null || "".equals(seqInfoResult)) {
                logger.info("获取双边协商-最新交易序列信息, 接口调用结果为空！");
                return;
            }
            Map<String, Object> seqInfoResultObj = JSON.parseObject(seqInfoResult, Map.class);
            List<TradeSeq> data = JSON.parseArray(JSON.toJSONString(seqInfoResultObj.get("data")), TradeSeq.class);
            if (data == null || data.size() == 0) {
                logger.info("获取双边协商-最新交易序列信息, data值为NULL或为空!  data:{}", data);
                return;
            }
            //添加入库时间
            String dataTimeStr = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            for (TradeSeq tradeSeq : data) {
                tradeSeq.setDataTime(dataTimeStr);
            }
            if (1 == DataPushUtils.isFormal(sgccUrl)) {
                //入正式环境数据对应数据表
                //删除旧数据
                tradeSeqDao.delTradeSeq();
                tradeSeqDao.pushTradeSeq(data);
                logger.info("获取双边协商-**正式环境**最新交易序列信息并入库成功!");
            } else {
                //入测试环境数据对应数据表
                tradeSeqDao.delTestTradeSeq();
                tradeSeqDao.pushTestTradeSeq(data);
                logger.info("获取双边协商-**测试环境**最新交易序列信息并入库成功!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("-调用双边协商-交易序列数据异常, 数据获取失败!");
        }
    }

    /**
     * 交互电力交易平台 ==  具体的交易公告详细信息
     *
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void getTradeNotice() {
        try {
            String sgccUrl = sgccAddressService.getSgccAddress(ENV_KEY);
            //调用交易公告详细信息接口
            String noticeResult = HttpUtils.sendPost(sgccUrl + "/dataPush/tradeNotice/selectTradeNotice", "", DataPushUtils.createToken(PUBLIC_KEY));
            logger.info("获取双边协商-具体交易公告详细信息, 接口调用结果 : {}", noticeResult);
            if (noticeResult == null || "".equals(noticeResult)) {
                logger.info("获取双边协商-具体交易公告详细信息, 接口调用结果为空！");
                return;
            }
            Map<String, Object> noticeResultObj = JSON.parseObject(noticeResult, Map.class);
            List<TradeNotice> noticeData = JSON.parseArray(JSON.toJSONString(noticeResultObj.get("data")), TradeNotice.class);
            if (noticeData == null || noticeData.size() == 0) {
                logger.info("获取双边协商-具体交易公告详细信息, data值为NULL或者为空! data: {}", noticeData);
                return;
            }
            //添加入库时间
            String localDateTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            for (TradeNotice tradeNotice : noticeData) {
                tradeNotice.setDataTime(localDateTime);
            }
            if (1 == DataPushUtils.isFormal(sgccUrl)) {
                //删除旧数据
                tradeNoticeDao.delNotice();
                tradeNoticeDao.pushTradeNotice(noticeData);
                logger.info("**双边协商-正式环境**交易公告详情数据入库成功!");
            } else {
                tradeNoticeDao.delTestNotice();
                tradeNoticeDao.pushTestTradeNotice(noticeData);
                logger.info("**双边协商-测试环境**交易公告详情数据入库成功!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("-调用双边协商-交易公告详细数据异常, 数据获取失败!");
        }
    }

    /**
     * 交互电力交易平台 == 获取交易序列下购售方双边申报明细数据 === 表格数据
     *
     * @throws Exception
     */
    @Override
    public List<Map<String, Object>> getDeclareDetailList(String seqId) {
        try {
            //调用购售方双边申报明细数据接口
            String declareDetailResult = HttpUtils.sendPost(sgccAddressService.getSgccAddress(ENV_KEY) + "/dataPush/tradeDeclareDetail/selectTradeDeclareDetailList", "seqId=" + seqId, DataPushUtils.createToken(PUBLIC_KEY));
            //String declareDetailResult = HttpUtils.sendPost("http://localhost:16002/getSource/getData", "seqId="+seqId, "");
//			String declareDetailResult = "{\r\n" +
//					"    \"code\": 1000,\r\n" +
//					"    \"msg\": \"操作成功!\",\r\n" +
//					"    \"data\": [\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 133.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"19:00-23:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"00:00-01:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 60.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"01:00-05:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"11:00-17:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 1500.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"23:00-24:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-03 10:00:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"19:00-23:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"00:00-01:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 60.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"01:00-05:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"11:00-17:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 1500.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"23:00-24:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-02 11:22:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"景泰新能寺滩风电有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃华电瓜州风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 133.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"19:00-23:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"00:00-01:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 60.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"01:00-05:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"11:00-17:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 1500.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"23:00-24:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2022-01-01 10:02:00\",\r\n" +
//					"            \"vendeeParticipantName\": \"甘肃大唐新能源有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"华能通渭风电有限责任公司\",\r\n" +
//					"            \"confirmEnergy\": 133.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"19:00-23:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"00:00-01:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 60.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"01:00-05:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"11:00-17:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 1500.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"23:00-24:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"金塔万晟光电有限公司金塔万晟100MW光伏\",\r\n" +
//					"            \"saleParticipantName\": \"中国铝业股份有限公司兰州分公司兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 133.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"19:00-23:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"00:00-01:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 60.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"01:00-05:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"11:00-17:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 1500.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"23:00-24:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-31 17:38:16\",\r\n" +
//					"            \"vendeeParticipantName\": \"恒远（金太阳）电力有限公司\",\r\n" +
//					"            \"saleParticipantName\": \"兰铝电厂\",\r\n" +
//					"            \"confirmEnergy\": 133.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"19:00-23:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"00:00-01:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 60.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"01:00-05:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"11:00-17:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 1500.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"17:00-18:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"23:00-24:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 0.00\r\n" +
//					"        },\r\n" +
//					"        {\r\n" +
//					"            \"tradeTimeRange\": \"09:00-11:00\",\r\n" +
//					"            \"confirmTime\": \"2021-12-30 12:32:14\",\r\n" +
//					"            \"vendeeParticipantName\": \"国网山东省电力公司\",\r\n" +
//					"            \"saleParticipantName\": \"甘肃靖远航天风力发电有限公司\",\r\n" +
//					"            \"confirmEnergy\": 133.00\r\n" +
//					"        }\r\n" +
//					"    ]\r\n" +
//					"}";
            //logger.info("获取双边协商-购售方双边申报明细数据, 接口调用结果: {}", declareDetailResult);
            if (declareDetailResult == null || "".equals(declareDetailResult)) {
                logger.info("获取双边协商-购售方双边申报明细数据, 接口调用结果为空！");
                return new ArrayList<>();
            }
            Map<String, Object> declareDetailObj = JSON.parseObject(declareDetailResult, Map.class);
            List<BuySaleDetail> declareDetailData = JSON.parseArray(JSON.toJSONString(declareDetailObj.get("data")), BuySaleDetail.class);
            System.out.println("数据量 :" + declareDetailData.size());

            if (declareDetailData == null || declareDetailData.size() == 0) {
                logger.info("获取购售方双边申报明细数据, data值为NUll或者为空! data: {}", declareDetailData);
                return new ArrayList<>();
            }
            //List<BuySaleDetail> dataSorted = declareDetailData.stream().sorted(Comparator.comparing(BuySaleDetail::getConfirmTime).reversed()).collect(Collectors.toList());
            Collections.sort(declareDetailData, new Comparator<BuySaleDetail>() {
                @Override
                public int compare(BuySaleDetail o1, BuySaleDetail o2) {
                    return o2.getConfirmTime().compareTo(o1.getConfirmTime());
                }
            });

            //数据量>300的话  限制展示300条
            List<BuySaleDetail> newList = declareDetailData.size() > 300 ? declareDetailData.subList(0, 300) : declareDetailData;
            System.out.println("newList.Size: " + newList.size());
            //封装结果
            List<Map<String, Object>> resultList = new LinkedList<>();
            for (BuySaleDetail buySaleDetail : newList) {
                Map<String, Object> rMap = new HashMap<>();
                rMap.put("saleParticipantName", buySaleDetail.getSaleParticipantName());
                rMap.put("vendeeParticipantName", buySaleDetail.getVendeeParticipantName());
                rMap.put("tradeTimeRange", buySaleDetail.getTradeTimeRange());
                rMap.put("confirmEnergy", new BigDecimal(buySaleDetail.getConfirmEnergy()).setScale(2, BigDecimal.ROUND_HALF_UP));
                rMap.put("confirmTime", buySaleDetail.getConfirmTime());
                resultList.add(rMap);
            }
            return resultList;
        } catch (Exception e) {
            e.printStackTrace();
            return new ArrayList<>();
        }
    }

    /**
     * 交互电力交易平台 == 获取售方申报信息和购方确认信息 - 交易情况数据
     *
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void getTradeDetailList() {
        try {
            String sgccUrl = sgccAddressService.getSgccAddress(ENV_KEY);
            //调用售方申报信息和购方确认信息接口
            String tradeDetailResult = HttpUtils.sendPost(sgccUrl + "/dataPush/tradeDetailInfo/selectTradeDetailList", "", DataPushUtils.createToken(PUBLIC_KEY));
            logger.info("获取双边协商-售方申报信息和购方确认信息, 接口调用结果: {}", tradeDetailResult);
            if (tradeDetailResult == null || "".equals(tradeDetailResult)) {
                logger.info("获取双边协商-售方申报信息和购方确认信息, 接口调用结果为空！");
                return;
            }
            Map<String, Object> tradeDetailObj = JSON.parseObject(tradeDetailResult, Map.class);
            List<TradeStatus> tradeDetailData = JSON.parseArray(JSON.toJSONString(tradeDetailObj.get("data")), TradeStatus.class);
            if (tradeDetailData == null || tradeDetailData.size() == 0) {
                logger.info("获取双边协商-售方申报信息和购方确认信息, data值为NUll或者为空! data: {}", tradeDetailData);
                return;
            }
            //添加入库时间
            String localDateTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            for (TradeStatus tradeStatus : tradeDetailData) {
                tradeStatus.setDataTime(localDateTime);
            }
            if (1 == DataPushUtils.isFormal(sgccUrl)) {
                //正式环境数据
                //删除旧数据
                tradeStatusDao.delStatus();
                tradeStatusDao.pushTradeStatus(tradeDetailData);
                logger.info("双边协商-交易情况**正式环境**数据入库成功!");
            } else {
                //测试环境数据
                tradeStatusDao.delTestStatus();
                tradeStatusDao.pushTestTradeStatus(tradeDetailData);
                logger.info("双边协商-交易情况**测试环境**数据入库成功!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("-调用双边协商-购售方申报数据异常, 数据获取失败!");
        }
    }

    /**
     * 交互电力交易平台 == 获取售方申报的电量信息统计 - 发电类型统计
     *
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void getTradeSaleEnergyList() {
        try {
            String sgccUrl = sgccAddressService.getSgccAddress(ENV_KEY);
            //调用售方申报的电量信息统计
            String tradeSaleResult = HttpUtils.sendPost(sgccUrl + "/dataPush/tradeSaleEnergy/selectTradeSaleEnergyList", "", DataPushUtils.createToken(PUBLIC_KEY));
            logger.info("获取双边协商-售方申报的电量信息统计, 接口调用结果: {}", tradeSaleResult);
            if (tradeSaleResult == null || "".equals(tradeSaleResult)) {
                logger.info("获取双边协商-售方申报的电量信息统计,接口调用结果为空！");
                return;
            }
            Map<String, Object> tradeSaleObj = JSON.parseObject(tradeSaleResult, Map.class);
            List<EnergyCensus> tradeSaleData = JSON.parseArray(JSON.toJSONString(tradeSaleObj.get("data")), EnergyCensus.class);
            if (tradeSaleData == null || tradeSaleData.size() == 0) {
                logger.info("获取双边协商-售方申报的电量信息统计, data值为NUll或者为空! data: {}", tradeSaleData);
                return;
            }
            //添加数据入库时间
            String localDateTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            for (EnergyCensus energyCensus : tradeSaleData) {
                energyCensus.setDataTime(localDateTime);
            }
            if (1 == DataPushUtils.isFormal(sgccUrl)) {
                //删除旧数据
                energyCensusDao.delEnergyCensus();
                energyCensusDao.pushEnergyCensus(tradeSaleData);
                logger.info("双边协商-售方申报的电量信息统计 - 发电类型统计**正式环境**数据入库成功!");
            } else {
                //删除旧数据
                energyCensusDao.delTestEnergyCensus();
                energyCensusDao.pushTestEnergyCensus(tradeSaleData);
                logger.info("双边协商-售方申报的电量信息统计 - 发电类型统计**测试环境**数据入库成功!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("-调用双边协商-发电类型统计数据异常, 数据获取失败!");
        }
    }

    /**
     * 交互电力交易平台 == 获取交易量价走势数据 - 实时数据
     *
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void getPriceTrend() {
        try {
            String sgccUrl = sgccAddressService.getSgccAddress(ENV_KEY);
            String priceTrendResult = HttpUtils.sendPost(sgccUrl + "/dataPush/tradeEnergyPriceLine/selectTradeEnergyPriceLineData", "", DataPushUtils.createToken(PUBLIC_KEY));
            logger.info("获取双边协商-交易量价走势-实时数据, 接口调用结果: {}", priceTrendResult);
            if (priceTrendResult == null || "".equals(priceTrendResult)) {
                logger.info("获取双边协商-交易量价走势-实时数据, 接口调用结果为空！");
                return;
            }
            Map<String, Object> priceTrendObj = JSON.parseObject(priceTrendResult, Map.class);
            List<PriceTrend> priceTrendData = JSON.parseArray(JSON.toJSONString(priceTrendObj.get("data")), PriceTrend.class);
            if (priceTrendData == null || priceTrendData.size() == 0) {
                logger.info("获取双边协商-交易量价走势-实时数据, data值为NUll或者为空! data: {}", priceTrendData);
                return;
            }
            String localDateTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            for (PriceTrend priceTrend : priceTrendData) {
                priceTrend.setDataTime(localDateTime);
            }
            if (1 == DataPushUtils.isFormal(sgccUrl)) {
                //删除旧数据
                priceTrendDao.delPriceTrend();
                //添加入库时间，并将申报时间转为yyyy-MM-dd格式
                priceTrendDao.pushPriceTrend(priceTrendData);
                logger.info("双边协商-交易量价走势数据-实时数据**正式环境**入库成功!");
            } else {
                //删除旧数据
                priceTrendDao.delTestPriceTrend();
                //添加入库时间，并将申报时间转为yyyy-MM-dd格式
                priceTrendDao.pushTestPriceTrend(priceTrendData);
                logger.info("双边协商-交易量价走势数据-实时数据**测试环境**入库成功!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("-调用双边协商-交易量价走势数据异常, 数据获取失败!");
        }
    }

    /**
     * 交互电力交易平台 == 获取交易量价走势 - 回演数据
     *
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void getPriceTrendReplay() {
        try {
            String sgccUrl = sgccAddressService.getSgccAddress(ENV_KEY);
            String priceTrendReplayResult = HttpUtils.sendPost(sgccUrl + "/dataPush/tradeEnergyPrice/tradeEnergyPriceFinishDate", "", DataPushUtils.createToken(PUBLIC_KEY));
            logger.info("获取双边协商-交易量价走势-回演数据, 接口调用结果: {}", priceTrendReplayResult);
            if (priceTrendReplayResult == null || "".equals(priceTrendReplayResult)) {
                logger.info("获取双边协商-交易量价走势-回演数据, 接口调用结果为空！");
                return;
            }
            Map<String, Object> priceTrendReplayObj = JSON.parseObject(priceTrendReplayResult, Map.class);
            List<PriceTrendReplay> priceTrendReplayData = JSON.parseArray(JSON.toJSONString(priceTrendReplayObj.get("data")), PriceTrendReplay.class);
            if (priceTrendReplayData == null || priceTrendReplayData.size() == 0) {
                logger.info("获取双边协商-交易量价走势-回演数据, data值为NUll或者为空! data: {}", priceTrendReplayData);
                return;
            }
            String localDateTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            for (PriceTrendReplay priceTrendReplay : priceTrendReplayData) {
                priceTrendReplay.setDataTime(localDateTime);
            }
            if (1 == DataPushUtils.isFormal(sgccUrl)) {
                //删除旧数据
                priceTrendReplayDao.delPriceTrendReplay();
                //添加入库时间，并将申报时间转为yyyy-MM-dd格式
                priceTrendReplayDao.pushPriceTrendReplay(priceTrendReplayData);
                logger.info("双边协商-交易量价走势-回演数据**正式环境**入库成功!");
            } else {
                priceTrendReplayDao.delTestPriceTrendReplay();
                priceTrendReplayDao.pushTestPriceTrendReplay(priceTrendReplayData);
                logger.info("双边协商-交易量价走势-回演数据**测试环境**入库成功!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("-调用双边协商-交易量价走势-回演数据异常, 数据获取失败!");
        }
    }

    /**
     * 交互电力交易平台 == 获取多日交易 - 实时状态 交易量价走势数据
     */
    @Override
    public void getRealTimeManyDay() {
        try {
            String sgccUrl = sgccAddressService.getSgccAddress(ENV_KEY);
            String manyDayResult = HttpUtils.sendPost(sgccUrl + "/dataPush/TradeEnergyMonth/selectEnergyMonth", "", DataPushUtils.createToken(PUBLIC_KEY));
            logger.info("双边协商-多日交易-实时状态 接口返回结果:{}", JSON.toJSONString(manyDayResult));
            if (StringUtils.isEmpty(manyDayResult)) {
                logger.info("双边协商-多日交易-实时状态, 返回结果为空!");
                return;
            }
            Map manyDayResMap = JSON.parseObject(manyDayResult, Map.class);
            //data数据 转 List
            List<ManyDayRealTime> manyDayResArray = JSON.parseArray(JSON.toJSONString(manyDayResMap.get("data")), ManyDayRealTime.class);
            if (manyDayResArray == null || manyDayResArray.size() == 0) return;
            System.out.println("---" + manyDayResArray);
            //数据入库
            if (1 == DataPushUtils.isFormal(sgccUrl)) {
                //删除旧数据
                realTimeManyDayDao.delRealTimeManyDay();
                //添加入库时间，并将申报时间转为yyyy-MM-dd格式
                realTimeManyDayDao.pushRealTimeManyDay(manyDayResArray);
                logger.info("双边协商-多日交易-实时状态 交易量价走势数据**正式环境**入库成功!");
            } else {
                realTimeManyDayDao.delTestRealTimeManyDay();
                realTimeManyDayDao.pushTestRealTimeManyDay(manyDayResArray);
                logger.info("双边协商-多日交易-实时状态 交易量价走势数据**测试环境**入库成功!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("-调用双边协商-多日交易-实时状态: 交易量价走势数据异常, 数据获取失败!");
        }
    }


    /**
     * 接口  ---- 获取所有交易序列数据
     */
    @Override
    public Map<String, Object> selectTradeSeqs(String tradeSeqType) {
        try {
            if (1 == DataPushUtils.isFormal(sgccAddressService.getSgccAddress(ENV_KEY))) {
                Map<String, Object> all = handleSeqList(tradeSeqDao.getAll(tradeSeqType));
                return all;
            } else {
                Map<String, Object> allTest = handleSeqList(tradeSeqDao.getAllTest(tradeSeqType));
                return allTest;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new HashMap<>();
        }
    }

    /**
     * 获取电力交易平台token
     */
    @Override
    public String getToken() {
        try {
            return DataPushUtils.createToken(PUBLIC_KEY);
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 预览交易公告pdf
     *
     * @return
     */
    @Override
    public Map<String, Object> getViewNotice(String tradeSeqId) throws IOException {
        //通过交易序列id查询已上传的pdf
        Map<String, Object> notice = uploadNoticeMapper.checkUpload(tradeSeqId);
        if (StringUtils.isEmpty(notice)) {
            return new HashMap<>();
        }
        //文件名称
        String uploadName = String.valueOf(notice.get("uploadName"));
        String fileName = uploadName.substring(0, uploadName.lastIndexOf("."));
        //文件预览地址
        String uploadPath = String.valueOf(notice.get("uploadPath"));
        if (StringUtils.isEmpty(uploadPath)) {
            return new HashMap<>();
        }
        //获取实际存储地址
        String realUploadPath = uploadPath.replaceAll("/notice/", NOTICE_FILE_PATH);
        //获取pdf文件每页大小, 总页数
		Map<String, Object> pdfInfo = getPdfInfo(realUploadPath);

		Map<String, Object> result = new HashMap<>();
		result.put("noticeWidth", pdfInfo.get("width"));
		result.put("noticeHeight", pdfInfo.get("height"));
        result.put("countPages", pdfInfo.get("countPages"));
        result.put("viewPath", uploadPath);
        result.put("fileName", fileName);
		return result;
    }

    /**
     * 获取市场建设情况-图片、文件列表信息
     * @return
     */
    @Override
    public List<Map<String, Object>> getMarketBuildFiles() throws IOException {
        List<Map<String, String>> marketList = uploadNoticeMapper.getMarketList(new HashMap<>());
        if (StringUtils.isEmpty(marketList)) {
            return new ArrayList<>();
        }
        List<Map<String, Object>> result = new ArrayList<>();
        for (Map<String, String> map : marketList) {
            String fileTitle = map.get("fileTitle");
            String fileUploadPath = map.get("fileUploadPath");
            String imageUploadPath = map.get("imageUploadPath");
            //获取实际上传地址
            String fileRealPath = fileUploadPath.replace("/marketUpload/", MARKET_FILE_PATH);
            Map<String, Object> pdfInfo = getPdfInfo(fileRealPath);

            Map<String, Object> resultItem = new HashMap<>();
            resultItem.put("fileName", fileTitle);
            resultItem.put("imagePath", imageUploadPath);
            resultItem.put("filePath", fileUploadPath);
            resultItem.put("noticeWidth", pdfInfo.get("width"));
            resultItem.put("noticeHeight", pdfInfo.get("height"));
            resultItem.put("countPages", pdfInfo.get("countPages"));
            result.add(resultItem);
        }
        return result;
    }

    /**
     * 获取pdf每页大小, 总页数
     */
    private Map<String, Object> getPdfInfo(String filePath) throws IOException {
        File file = new File(filePath);
        PdfReader pdfReader = new PdfReader(new FileInputStream(file));
        //获取总页数
        int countPages = pdfReader.getNumberOfPages();
        //页宽
        float width = pdfReader.getPageSize(1).getWidth();
        //页高
        float height = pdfReader.getPageSize(1).getHeight();
        //单位转换
        //pdf测量单位point  1pt=1/72 inch  1英寸=25.4毫米
        //转为毫米: pt*25.4/72
		double widthCm = width * 25.4 / 72 / 10;
		double heightCm = height * 25.4 / 72 / 10;
		Map<String, Object> res = new HashMap<>();
		res.put("countPages", countPages);
		res.put("width", BigDecimal.valueOf(widthCm).setScale(3, BigDecimal.ROUND_HALF_DOWN));
		res.put("height", BigDecimal.valueOf(heightCm).setScale(3, BigDecimal.ROUND_HALF_DOWN));
		return res;
    }
}
