package com.alonginfo.project.gansu.sync.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.alonginfo.common.utils.DateUtils;
import com.alonginfo.common.utils.StringUtils;
import com.alonginfo.common.utils.http.HttpUtils;
import com.alonginfo.project.gansu.mapper.MarketBuildDataSyncMapper;
import com.alonginfo.project.gansu.service.SgccAddressService;
import com.alonginfo.project.gansu.sync.MarketBuildDataSync;
import com.alonginfo.utils.DataPushUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;

/**
 * @author: HaoXs
 * @Date: 2023/1/5 09:49
 * @Description:  市场建设 远程接口调用
 */
@Service
@Slf4j
public class MarketBuildDataSyncImpl implements MarketBuildDataSync {

    /**
     * 市场建设情况场景-年度电力直接交易规模URL
     */
    public static String powerDirectDealUrl = "/clearingDate/selectClearingByYear";

    /**
     * 市场建设情况场景-各类市场主体数量接口URL
     */
    public static String everyTypeSubjectNumUrl = "/market/getMarketPropertion";

    @Value("${env.key}")
    private String ENV_KEY;

    @Value("${sgcc.publicKey}")
    private String publicKey;

    @Autowired
    private SgccAddressService sgccAddressService;

    @Autowired
    private MarketBuildDataSyncMapper marketBuildDataSyncMapper;

    /**
     * 年度电力直接交易规模
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void powerDirectDeal() {
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);
        String powerDirectDealResStr = HttpUtils.sendPost(sgccAddress + powerDirectDealUrl, "", DataPushUtils.createToken(publicKey));
        log.info("-市场建设情况-年度电力直接交易规模接口调用结果: {}", powerDirectDealResStr);
        if (StringUtils.isEmpty(powerDirectDealResStr)) {
            log.error("-市场建设情况-年度电力直接交易规模接口调用结果为空! url: {}, result: {}", powerDirectDealUrl, powerDirectDealResStr);
            return;
        }
        Map map = JSON.parseObject(powerDirectDealResStr, Map.class);
        Object data = map.get("data");
        if (null == data) {
            log.error("-市场建设情况-年度电力直接交易规模接口, data值为NULL!");
        }
        List<Map<String, String>> dataRes = JSON.parseObject(JSON.toJSONString(data), new TypeReference<List<Map<String, String>>>(){});
        //先删除再入库
        marketBuildDataSyncMapper.delPowerDirectDeal();
        marketBuildDataSyncMapper.insertPowerDirectDeal(dataRes);
    }

    /**
     * 各类市场主体数量接口
     * 需要传参 year 获取当年数据
     * 页面默认展示以往6年数据  循环调用6年
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void everyTypeSubjectNum() throws IOException {
        //获取以往6年年份(包括当年)
        List<String> lastYear = DateUtils.getLastYear(6);
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);

        //循环获取每年数据结果集合
        List<Map<String, String>> yearDataRes = new LinkedList<>();
        for (String year : lastYear) {
            Map<String, String> jsonParam = new HashMap<>();
            jsonParam.put("year", year);
            String everyTypeSubjectResStr = HttpUtils.sendPostDataByJson(sgccAddress + everyTypeSubjectNumUrl, JSON.toJSONString(jsonParam), DataPushUtils.createToken(publicKey));
            log.info("-市场建设情况-各类市场主体数量接口调用结果: {}", everyTypeSubjectResStr);
            if (StringUtils.isEmpty(everyTypeSubjectResStr)) {
                log.error("-市场建设情况-各类市场主体数量接口调用结果为空! url: {}, param: {}, result: {}", everyTypeSubjectNumUrl, year, everyTypeSubjectResStr);
                continue;
            }
            Map map = JSON.parseObject(everyTypeSubjectResStr, Map.class);
            Object data = map.get("data");
            if (null == data) {
                log.error("-市场建设情况-各类市场主体数量接口, data值为空! params: {}", year);
                continue;
            }
            List<Map<String, String>> dataResList = JSON.parseObject(JSON.toJSONString(data), new TypeReference<List<Map<String, String>>>() {});
            //membertype-企业类型 count-注册数 tranCount-参与交易数 
            //10 发电企业
            String power = dataResList.stream().filter(f -> "10".equals(f.get("membertype"))).map(m -> m.get("count")).findFirst().orElse(null);
            //20 电力用户
            String user = dataResList.stream().filter(f -> "20".equals(f.get("membertype"))).map(m -> m.get("count")).findFirst().orElse(null);
            //30 售电公司
            String sale = dataResList.stream().filter(f -> "30".equals(f.get("membertype"))).map(m -> m.get("count")).findFirst().orElse(null);
            //封装格式处理后的数据
            Map<String, String> yearRes = new HashMap<>();
            yearRes.put("year", year);
            yearRes.put("power", power);
            yearRes.put("sale", sale);
            yearRes.put("user", user);
            yearDataRes.add(yearRes);
        }
        if (CollectionUtils.isEmpty(yearDataRes)) {
            return;
        }
        //先删除再入库
        marketBuildDataSyncMapper.delEveryTypeSubject();
        marketBuildDataSyncMapper.insertEveryTypeSubject(yearDataRes);
    }
}
