package com.alonginfo.project.gansu.sync.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.alonginfo.common.utils.StringUtils;
import com.alonginfo.common.utils.http.HttpUtils;
import com.alonginfo.project.gansu.mapper.MarketSurveyDataSyncMapper;
import com.alonginfo.project.gansu.service.SgccAddressService;
import com.alonginfo.project.gansu.sync.MarketSurveyDataSync;
import com.alonginfo.utils.DataPushUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.time.YearMonth;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author: HaoXs
 * @Date: 2023/1/4 14:57
 * @Description: 市场概况 远程接口调用实现
 */
@Service
@Slf4j
public class MarketSurveyDataSyncImpl implements MarketSurveyDataSync {

    /**
     * 市场主体类型、数量、占比接口
     */
    public static String marketSubjectUrl = "/market/getMarketMemberType";

    /**
     * 发电企业类型、数量、占比接口
     */
    public static String yieldEnterpriseUrl = "/mmBuild/powerRatio";

    @Value("${env.key}")
    private String ENV_KEY;

    @Value("${sgcc.publicKey}")
    private String publicKey;

    @Autowired
    private SgccAddressService sgccAddressService;

    @Autowired
    private MarketSurveyDataSyncMapper marketSurveyDataSyncMapper;

    /**
     * 当前市场主体情况-市场主体类型、数量、占比接口
     * @return
     */
    @Override
    public void getMarketSubjectData() throws IOException {
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);
        int year = YearMonth.now().getYear();
        Map<String, String> requestParam = new HashMap<>();
        requestParam.put("year", String.valueOf(year));
        String marketSubjectResultStr = HttpUtils.sendPostDataByJson(sgccAddress+marketSubjectUrl, JSON.toJSONString(requestParam), DataPushUtils.createToken(publicKey));
        log.info("-当前市场主体情况-市场主体类型、数量、占比接口结果: {}", marketSubjectResultStr);
        if (StringUtils.isEmpty(marketSubjectResultStr)) {
            log.error("-当前市场主体情况-市场主体类型、数量、占比接口为空!, 接口地址: {}, result: {}", marketSubjectUrl, marketSubjectResultStr);
            return;
        }
        Map map = JSON.parseObject(marketSubjectResultStr, Map.class);
        Object dataRes = map.get("data");
        if (null == dataRes) {
            log.error("-当前市场主体情况-市场主体类型、数量、占比接口, data值为空或为NULL!");
            return;
        }
        List<Map<String, Object>> dataList = JSON.parseObject(JSON.toJSONString(dataRes), new TypeReference<List<Map<String, Object>>>() {});
        //返回格式  10：发电企业，20：电力用户，30：售电公司
        //{
        //	"status": 0,
        //	"message": "数据查询成功",
        //	"data ": [{
        //		"membertype": "10",
        //		"count": 1208,
        //		"tradeCount": 0,
        //		"proportion": null
        //	}, {
        //		"membertype": "20",
        //		"count": 3804,
        //		"tradeCount": 0,
        //		"proportion": null
        //	}, {
        //		"membertype": "30",
        //		"count": 47,
        //		"tradeCount": 0,
        //		"proportion": null
        //	}]
        //}
        //发电企业家数
        Integer yieldNum = dataList.stream().filter(f -> "10".equals(f.get("membertype"))).map(m -> {
            return Integer.parseInt(m.get("count").toString());
        }).findFirst().orElse(0);
        // 电力用户数
        Integer userNum = dataList.stream().filter(f -> "20".equals(f.get("membertype"))).map(m -> {
            return Integer.parseInt(m.get("count").toString());
        }).findFirst().orElse(0);
        //售电公司数
        Integer saleNum = dataList.stream().filter(f -> "30".equals(f.get("membertype"))).map(m -> {
            return Integer.parseInt(m.get("count").toString());
        }).findFirst().orElse(0);

        //占比
        BigDecimal yieldDecimal = BigDecimal.valueOf(yieldNum);
        BigDecimal userDecimal = BigDecimal.valueOf(userNum);
        BigDecimal saleDecimal = BigDecimal.valueOf(saleNum);
        BigDecimal countDecimal = BigDecimal.valueOf((yieldNum + userNum + saleNum));

        BigDecimal yieldRatio = yieldDecimal.divide(countDecimal, 2, BigDecimal.ROUND_HALF_UP);
        BigDecimal userRatio = userDecimal.divide(countDecimal, 2, BigDecimal.ROUND_HALF_UP);
        BigDecimal saleRatio = saleDecimal.divide(countDecimal, 2, BigDecimal.ROUND_HALF_UP);

        Map<String, Object> dataParam = new HashMap<>();
        dataParam.put("powerCount", yieldNum);
        dataParam.put("userCount", userNum);
        dataParam.put("saleCount", saleNum);
        dataParam.put("power", yieldRatio);
        dataParam.put("user", userRatio);
        dataParam.put("sale", saleRatio);
        //数据入库
        marketSurveyDataSyncMapper.insertMarketSubject(dataParam);
    }

    /**
     * 当前市场主体情况-发电企业类型、数量、占比接口
     * @return
     */
    @Override
    public void getYieldEnterpriseData() {
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);
        String yieldEnterpriseResultStr = HttpUtils.sendPost(sgccAddress+yieldEnterpriseUrl, "", DataPushUtils.createToken(publicKey));
        log.info("-当前市场主体情况-发电企业类型、数量、占比接口结果: {}", yieldEnterpriseResultStr);
        if (StringUtils.isEmpty(yieldEnterpriseResultStr)) {
            log.error("-当前市场主体情况-发电企业类型、数量、占比接口为空!, 接口地址: {}, result: {}", yieldEnterpriseUrl, yieldEnterpriseResultStr);
            return;
        }
        Map map = JSON.parseObject(yieldEnterpriseResultStr, Map.class);
        Map<String, Object> dataRes = (Map<String, Object>)map.get("data");
        if (StringUtils.isEmpty(dataRes)) {
            log.error("-当前市场主体情况-发电企业类型、数量、占比接口, data值为空或为NULL");
            return;
        }
        marketSurveyDataSyncMapper.insertYieldEnterprise(dataRes);
    }

//    /**
//     * 市场主体参与情况
//     * @return
//     */
//    @Override
//    public String getMarketSubjectJoin() {
//        long beginTime = System.currentTimeMillis();
//        String marketSubjectJoinResultStr = HttpUtils.sendPost(marketSubjectJoinUrl, "", DataPushUtils.createToken(publicKey));
//        log.info("-市场主体参与情况：{}", marketSubjectJoinResultStr);
//        if (StringUtils.isEmpty(marketSubjectJoinResultStr)) {
//            log.info("-市场主体参与情况接口为空!, 接口地址: {}, result: {}", marketSubjectUrl, marketSubjectJoinResultStr);
//            return "";
//        }
//        long endTime = System.currentTimeMillis();
//        log.info("-市场主体参与情况接口耗时: {}", (endTime - beginTime)/1000);
//        return marketSubjectJoinResultStr;
//    }
}
