package com.alonginfo.project.gansu.sync.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.alonginfo.common.utils.DateUtils;
import com.alonginfo.common.utils.StringUtils;
import com.alonginfo.common.utils.http.HttpUtils;
import com.alonginfo.project.gansu.mapper.ProvinceInnerDataSyncMapper;
import com.alonginfo.project.gansu.service.SgccAddressService;
import com.alonginfo.project.gansu.sync.ProvinceInnerDataSync;
import com.alonginfo.utils.DataPushUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.time.Year;
import java.time.YearMonth;
import java.util.*;

/**
 * @author: HaoXs
 * @Date: 2023/5/12 10:13
 * @Description:  省内交易情况 数据同步
 */
@Service
@Slf4j
public class ProvinceInnerDataSyncImpl implements ProvinceInnerDataSync {

    /**
     * 交易组织情况
     */
    private static String dealOrganizeUrl = "/clearingDate/selectOrgTrade";

    /**
     * 市场主体参与情况
     */
    private static String provinceInnerSubjectJoinUrl = "/clearingDate/selectPxDetails";

    /**
     * 各电源类型交易电量占比
     */
    private static String cleanEnergyDealPowerUrl = "/tradeSituation/selectCleanEnergyList";

    /**
     * 当年交易电量电价走势
     */
    private static String powerPriceTrendUrl = "/tradeEnergyPriceTrend/selectTradeEnergyPriceTrendDataByYear";

    /**
     * 交易概览
     */
    private static String dealSurveyProvinceInnerUrl = "/delivery/selectTradeInfo";

    @Value("${env.key}")
    private String ENV_KEY;

    @Value("${sgcc.publicKey}")
    private String publicKey;

    @Autowired
    private SgccAddressService sgccAddressService;

    @Autowired
    private ProvinceInnerDataSyncMapper provinceInnerDataSyncMapper;

    /**
     * 交易组织情况接口
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void dealOrganize() {
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);
        String dealOrganizeResStr = HttpUtils.sendPost(sgccAddress + dealOrganizeUrl, "", DataPushUtils.createToken(publicKey));
        log.info("-省内交易情况-交易组织情况接口调用结果: {}", dealOrganizeResStr);
        if (StringUtils.isEmpty(dealOrganizeResStr)) {
            log.error("-省内交易情况-交易组织情况接口调用结果为空! url:{}, param:{}, result:{}", dealOrganizeUrl, "", dealOrganizeResStr);
            return;
        }
        Map map = JSON.parseObject(dealOrganizeResStr, Map.class);
        Object data = map.get("data");
        if (null == data) {
            log.error("-省内交易情况-交易组织情况接口, data值为NULL!");
            return;
        }
        List<Map<String, String>> dataResList = JSON.parseObject(JSON.toJSONString(data), new TypeReference<List<Map<String, String>>>() {});
        //封装入库参数
        List<Map<String, String>> insertParamList = new ArrayList<>();
        //tradeYear 交易年份 tradeCount1 双边组织次数 tradeEnergy1 双边交易电量  2、3、4 集中、挂牌、D+3
        dataResList.forEach(mapObj -> {
            Map<String, String> insertParamItem = new HashMap<>();
            insertParamItem.put("tradeYear", mapObj.get("tradeYear"));
            insertParamItem.put("sbNum", mapObj.get("tradeCount1"));
            insertParamItem.put("jzNum", mapObj.get("tradeCount2"));
            insertParamItem.put("gpNum", mapObj.get("tradeCount3"));
            insertParamItem.put("d3Num", mapObj.get("tradeCount4"));
            insertParamItem.put("sbEnergy", mapObj.get("tradeEnergy1"));
            insertParamItem.put("jzEnergy", mapObj.get("tradeEnergy2"));
            insertParamItem.put("gpEnergy", mapObj.get("tradeEnergy3"));
            insertParamItem.put("d3Energy", mapObj.get("tradeEnergy4"));
            insertParamList.add(insertParamItem);
        });
        if (CollectionUtils.isEmpty(insertParamList)) {
            return;
        }
        //先删除再入库
        provinceInnerDataSyncMapper.delDealOrganize();
        provinceInnerDataSyncMapper.insertDealOrganize(insertParamList);
    }

    /**
     * 当年市场主体参与情况
     */
    @Override
    public void provinceInnerSubjectJoin() throws IOException {
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);
        //获取当年、前一年的年份
        List<String> years = DateUtils.getLastYear(2);

        //当年、前一年接口返回数据
        List<Map<String, String>> previousYearData = new ArrayList<>();
        for (String year : years) {
            Map<String, String> requestParam = new HashMap<>();
            requestParam.put("tradeYear", year);
            String subjectJoinResStr = HttpUtils.sendPostDataByJson(sgccAddress + provinceInnerSubjectJoinUrl, JSON.toJSONString(requestParam), DataPushUtils.createToken(publicKey));
            log.info("-省内交易情况-当年市场主体参与情况接口调用结果: {}", subjectJoinResStr);
            if (StringUtils.isEmpty(subjectJoinResStr)) {
                log.error("-省内交易情况-当年市场主体参与情况接口调用结果为空! url:{}, param:{}, result:{}", provinceInnerSubjectJoinUrl, requestParam, subjectJoinResStr);
                continue;
            }
            Map map = JSON.parseObject(subjectJoinResStr, Map.class);
            Object data = map.get("data");
            if (null == data) {
                log.error("-省内交易情况-当年市场主体参与情况, data值为NULL!");
                continue;
            }
            //处理data
            //orgTrade 市场准入数据  joinTrade 参与交易数据
            //participantType: 10，发电企业，20售电公司，30 电力用户
            Map<String, List<Map<String, String>>> dataResMap = JSON.parseObject(JSON.toJSONString(data), new TypeReference<Map<String, List<Map<String, String>>>>() {
            });
            List<Map<String, String>> orgTrade = dataResMap.get("orgTrade");
            List<Map<String, String>> joinTrade = dataResMap.get("joinTrade");
            //市场准入
            String orgYieldNum = null;
            String orgSaleNum = null;
            String orgUserNum = null;
            if (StringUtils.isNotEmpty(orgTrade)) {
                orgYieldNum = orgTrade.stream().filter(obj -> "10".equals(obj.get("participantType"))).map(m -> m.get("tradeCount")).findFirst().orElse(null);
                orgSaleNum = orgTrade.stream().filter(obj -> "20".equals(obj.get("participantType"))).map(m -> m.get("tradeCount")).findFirst().orElse(null);
                orgUserNum = orgTrade.stream().filter(obj -> "30".equals(obj.get("participantType"))).map(m -> m.get("tradeCount")).findFirst().orElse(null);
            }
            //参与交易
            String joinYieldNum = null;
            String joinSaleNum = null;
            String joinUserNum = null;
            if (StringUtils.isNotEmpty(joinTrade)) {
                joinYieldNum = joinTrade.stream().filter(obj -> "10".equals(obj.get("participantType"))).map(m -> m.get("tradeCount")).findFirst().orElse(null);
                joinSaleNum = joinTrade.stream().filter(obj -> "20".equals(obj.get("participantType"))).map(m -> m.get("tradeCount")).findFirst().orElse(null);
                joinUserNum = joinTrade.stream().filter(obj -> "30".equals(obj.get("participantType"))).map(m -> m.get("tradeCount")).findFirst().orElse(null);
            }
            //入库参数
            Map<String, String> insertParam = new HashMap<>();
            insertParam.put("tradeYear", year);
            insertParam.put("orgYield", orgYieldNum);
            insertParam.put("orgSale", orgSaleNum);
            insertParam.put("orgUser", orgUserNum);
            insertParam.put("joinYield", joinYieldNum);
            insertParam.put("joinSale", joinSaleNum);
            insertParam.put("joinUser", joinUserNum);
            previousYearData.add(insertParam);
        }
        if (CollectionUtils.isEmpty(previousYearData)) {
            return;
        }
        provinceInnerDataSyncMapper.delSubjectJoin();
        provinceInnerDataSyncMapper.insertSubjectJoin(previousYearData);
    }

    /**
     * 各电源类型交易电量占比
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void cleanEnergyDealPower() throws IOException {
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);

        Map<String, String> requestParam = new HashMap<>();
        requestParam.put("tradeYear", String.valueOf(Year.now().getValue()));
        String cleanEnergyResStr = HttpUtils.sendPostDataByJson(sgccAddress + cleanEnergyDealPowerUrl, JSON.toJSONString(requestParam), DataPushUtils.createToken(publicKey));
        log.info("-省内交易情况-清洁能源成交电量接口调用结果: {}", cleanEnergyResStr);
        if (StringUtils.isEmpty(cleanEnergyResStr)) {
            log.error("-省内交易情况-清洁能源成交电量接口调用结果为空! url:{}, param:{}, result:{}", provinceInnerSubjectJoinUrl, requestParam, cleanEnergyResStr);
            return;
        }
        Map map = JSON.parseObject(cleanEnergyResStr, Map.class);
        Object data = map.get("data");
        if (null == data) {
            log.error("-省内交易情况-清洁能源成交电量, data值为NULL");
            return;
        }
        List<Map<String, String>> dataResList = JSON.parseObject(JSON.toJSONString(data), new TypeReference<List<Map<String, String>>>() {});

        //封装入库参数
        List<Map<String, String>> insertParam = new ArrayList<>();
        dataResList.forEach(item -> {
            Map<String, String> insertParamItem = new HashMap<>();
            insertParamItem.put("tradeYear", item.get("tradeYear"));
            insertParamItem.put("energyType", item.get("energyType"));
            insertParamItem.put("vendee", item.get("sumVendeeEnergy"));
            insertParamItem.put("sale", item.get("sumSaleEnergy"));
            insertParam.add(insertParamItem);
        });
        if (CollectionUtils.isEmpty(insertParam)) {
            return;
        }
        provinceInnerDataSyncMapper.delCleanEnergy();
        provinceInnerDataSyncMapper.insertCleanEnergy(insertParam);
    }

    /**
     * 当年交易电量电价趋势
     */
    @Override
    public void powerPriceTrend() throws IOException {
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);
        Map<String, String> requestMap = new HashMap<>();
        requestMap.put("tradeYear", String.valueOf(Year.now().getValue()));
        String trendResStr = HttpUtils.sendPostDataByJson(sgccAddress + powerPriceTrendUrl, JSON.toJSONString(requestMap), DataPushUtils.createToken(publicKey));
        log.info("-省内交易情况-当年交易电量电价趋势接口调用结果: {}", trendResStr);
        if (StringUtils.isEmpty(trendResStr)) {
            log.error("-省内交易情况-当年交易电量电价趋势接口调用结果为空! url:{}, param:{}, result:{}", powerPriceTrendUrl, requestMap, trendResStr);
            return;
        }
        Map map = JSON.parseObject(trendResStr, Map.class);
        Object data = map.get("data");
        if (null == data) {
            log.error("-省内交易情况-当年交易电量电价趋势, data值为NULL");
            return;
        }
        List<Map<String, String>> dataResList = JSON.parseObject(JSON.toJSONString(data), new TypeReference<List<Map<String, String>>>() {});

        List<Map<String, String>> insertList = new ArrayList<>();
        dataResList.forEach(item -> {
            Map<String, String> insertListItem = new HashMap<>();
            insertListItem.put("tradeMonth", item.get("tradeMonth"));
            insertListItem.put("vendeeEnergy", item.get("sumVendeeEnergy"));
            insertListItem.put("vendeePrice", item.get("avgVendeePrice"));
            insertListItem.put("saleEnergy", item.get("sumSaleEnergy"));
            insertListItem.put("salePrice", item.get("avgSalePrice"));
            insertList.add(insertListItem);
        });
        if (CollectionUtils.isEmpty(dataResList)) {
            return;
        }
        provinceInnerDataSyncMapper.delPowerPriceTrend();
        provinceInnerDataSyncMapper.insertPowerPriceTrend(insertList);
    }

    /**
     * 省内交易概况-交易概览
     */
    @Override
    public void dealSurveyProvinceInner() throws IOException {
        String sgccAddress = sgccAddressService.getSgccAddress(ENV_KEY);
        System.out.println("sgccAddress = " + sgccAddress);
        //当年年份、去年年份
        List<String> years = DateUtils.getLastYear(2);

        for (String year : years) {
            Map<String, String> param = new HashMap<>();
            param.put("year", year);
            String dealSurveyResStr = HttpUtils.sendPostDataByJson(sgccAddress + dealSurveyProvinceInnerUrl, JSON.toJSONString(param), DataPushUtils.createToken(publicKey));
            log.info("-省内交易情况-交易概览接口调用结果: param:{}, result:{}", year, dealSurveyResStr);
            if (StringUtils.isEmpty(dealSurveyResStr)) {
                log.error("-省内交易情况-交易概览接口调用结果为空! url:{}, param:{}, result:{}",
                        dealSurveyProvinceInnerUrl, JSON.toJSONString(param), dealSurveyResStr);
                continue;
            }
            Map map = JSON.parseObject(dealSurveyResStr, Map.class);
            Object data = map.get("data");
            if (null == data) {
                log.error("-省内交易情况-交易概览, param: {}, data值为NULL!", year);
                continue;
            }
            List<Map<String, Object>> dealSurveyResList = JSON.parseObject(JSON.toJSONString(data), new TypeReference<List<Map<String, Object>>>() {});

            List<Map<String, Object>> insertList = new ArrayList<>();
            for (Map<String, Object> dealSurveyResMap : dealSurveyResList) {
                Map<String, Object> insertParam = new HashMap<>();
                insertParam.put("tradeYear", year);
                insertParam.put("countTrade", dealSurveyResMap.get("tradeCount"));
                insertParam.put("sumVendeeEnergy", dealSurveyResMap.get("energy"));
                insertList.add(insertParam);
            }
            if (CollectionUtils.isEmpty(insertList)) {
                continue;
            }
            provinceInnerDataSyncMapper.delDealSurvey(year);
            provinceInnerDataSyncMapper.insertDealSurvey(insertList);
        }

    }
}
