package com.alonginfo.utils;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * 双边协商、集中竞价、挂牌交易、D+3滚动交易 公共方法
 * @author Administrator
 *
 */
public class DataPushUtils {

	private static final Logger logger = LoggerFactory.getLogger(DataPushUtils.class);
	
	/**
	 * 电力交易平台测试环境ip
	 */
	private static final String TEST_IP = "25.215.196.155";
	/**
	 * 电力交易平台生产环境ip
	 */
	private static final String PROD_IP = "25.215.196.215";
	
	/**
	 * 生成校验token
	 * @throws Exception 
	 */
	public static String createToken(String publicKey){
		try {
			//约定传递字符串gspx+当天日期
			String dateTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMdd"));
			String key = "gspx" + dateTime;
			//RSA加密
			String encryptStr = RSAUtil.encrypt(key, publicKey);
			return encryptStr;
		}catch (Exception e) {
			e.printStackTrace();
			logger.info("-生成token密钥异常!");
			return "";
		}

	}
	
	/**
	 * 判断当前电力交易平台为测试环境还是正式环境
	 * @return 1-生产 0-测试
	 */
	public static int isFormal(String ip) {
		if (ip.contains(PROD_IP)) {
			return 1;
		}
		return 0;
	}
	
	/**
	 * 生成pdf文件映射地址
	 * @return
	 */
	private static String trimPdfAddress(String address) {
		String fileString = address;
		String substring = fileString.substring(fileString.lastIndexOf("/"), fileString.length());
		String result = "views" +substring;
		return result;
	}

	/**
	 * 处理交易序列返回格式
	 * List<Map<>  ->   Map<>{list:[] 交易序列集合;  arr:[] 交易序列id集合}
	 */
	public static Map<String, Object> handleSeqList(List<Map<String, String>> seqList) {
		try {
			List<String> seqIds = new ArrayList<>();
			for (Map<String, String> seq : seqList) {
				seqIds.add(seq.get("tradeSeqId"));
			}

			Map<String, Object> result = new HashMap<>();
			result.put("list", seqList);
			result.put("arr", seqIds);
			return result;
		} catch (Exception e) {
			e.printStackTrace();
			logger.info("处理交易序列数据失败!");
			return new HashMap<>();
		}
	}

	/**
	 * 	处理购售方确认信息 - 表格数据
	 * @param buySaleDetails
	 */
	public static List<LinkedList<String>> dealBuySaleDetail(List<Map<String, Object>> buySaleDetails) {
		if (buySaleDetails == null || buySaleDetails.size() == 0) {
			return new ArrayList<LinkedList<String>>();
		}
		List<LinkedList<String>> resultList = new LinkedList<LinkedList<String>>();
		for (Map<String, Object> map : buySaleDetails) {
			LinkedList<String> linkedList = new LinkedList<String>();
			linkedList.add(map.get("saleParticipantName") == null ? "" : String.valueOf(map.get("saleParticipantName")));
			linkedList.add(map.get("vendeeParticipantName") == null ? "" : String.valueOf(map.get("vendeeParticipantName")));
			linkedList.add(map.get("tradeTimeRange") == null ? "" : String.valueOf(map.get("tradeTimeRange")));
			linkedList.add(map.get("confirmEnergy") == null ? "" : String.valueOf(map.get("confirmEnergy")));
			linkedList.add(map.get("confirmTime") == null ? "" : String.valueOf(map.get("confirmTime")));
			resultList.add(linkedList);
		}
		return resultList;
	}

	/**
	 * 处理实时状态-多日交易 交易量价走势
	 * @param manyDayList
	 * @return
	 */
	public static Map<String, List<String>> handleRealTimeManyDay(List<Map<String, String>> manyDayList) {
		try {
			if (manyDayList == null || manyDayList.size() == 0) return new HashMap<>();
			List<String> xAxis = new LinkedList<>();
			List<String> value = new LinkedList<>();
			List<String> price = new LinkedList<>();
			for (Map<String, String> map : manyDayList) {
				xAxis.add(map.get("createTime"));
				value.add(map.get("sumEnergy"));
				price.add(map.get("avgPrice"));
			}
			Map<String, List<String>> result = new HashMap();
			result.put("xAxis", xAxis);
			result.put("value", value);
			result.put("price", price);
			return result;
		} catch (Exception e) {
			e.printStackTrace();
			return new HashMap<>();
		}
	}

	/**
	 * 处理交易量价走势数据 - 实时数据
	 * @param trends
	 * @return
	 */
	public static Map<String, Object> dealTrends(List<Map<String, String>> trends) {
		if (trends==null || trends.size()==0) {
			return new HashMap<>();
		}
		try {
			List<String> xAxisData = new LinkedList<>();
			//峰值
			List<BigDecimal> peak = new LinkedList<>();
			//平值
			List<BigDecimal> flat = new LinkedList<>();
			//谷值
			List<BigDecimal> valley = new LinkedList<>();
			//峰端均价
			List<BigDecimal> peakAvg = new LinkedList<>();
			//平段均价
			List<BigDecimal> flatAvg = new LinkedList<>();
			//谷段均价
			List<BigDecimal> valleyAvg = new LinkedList<>();
			for (Map<String, String> map : trends) {
				xAxisData.add(String.valueOf(map.get("reportTime")));
				peak.add(new BigDecimal(String.valueOf(map.get("peakEnergy"))));
				flat.add(new BigDecimal(String.valueOf(map.get("flatEnergy"))));
				valley.add(new BigDecimal(String.valueOf(map.get("valleyEnergy"))));
				peakAvg.add(new BigDecimal(String.valueOf(map.get("peakAvgValue"))));
				flatAvg.add(new BigDecimal(String.valueOf(map.get("flatAvgValue"))));
				valleyAvg.add(new BigDecimal(String.valueOf(map.get("valleyAvgValue"))));
			}
			//封装返回结果
			Map<String, Object> resultMap = new LinkedHashMap<String, Object>();
			resultMap.put("peak", peak);
			resultMap.put("flat", flat);
			resultMap.put("valley", valley);
			resultMap.put("peakAvg", peakAvg);
			resultMap.put("flatAvg", flatAvg);
			resultMap.put("valleyAvg", valleyAvg);
			resultMap.put("xAxisData", xAxisData);
			return resultMap;
		} catch (Exception e) {
			e.printStackTrace();
			return new HashMap<>();
		}
	}

	/**
	 * 处理交易量价走势数据 - 回演数据
	 * @param trendReplayBySeqId
	 * @return
	 */
	public static Map<String, Object> dealTrendReplays(List<Map<String,  String>> trendReplayBySeqId) {
		try {
			if (trendReplayBySeqId == null || trendReplayBySeqId.size() == 0) {
				return new HashMap<>();
			}
			List<String> xAxisData = new LinkedList<>();
			List<BigDecimal> countEnergy = new LinkedList<>();
			List<BigDecimal> peakAvg = new LinkedList<>();
			List<BigDecimal> flatAvg = new LinkedList<>();
			List<BigDecimal> valleyAvg = new LinkedList<>();
			//按照峰、平、谷排序
			for (Map<String, String> map : trendReplayBySeqId) {
				String timeDivisionNmae = Optional.ofNullable(map.get("timeDivisionNmae")).orElse("");

				xAxisData.add(map.get("tradeTimeRange"));
				countEnergy.add(new BigDecimal(String.valueOf(map.get("countEnergy"))).setScale(2, BigDecimal.ROUND_HALF_UP));
				if (timeDivisionNmae.contains("峰")) {
					peakAvg.add(new BigDecimal(String.valueOf(map.get("avgValue"))).setScale(2, BigDecimal.ROUND_HALF_UP));
					flatAvg.add(null);
					valleyAvg.add(null);
				} else if (timeDivisionNmae.contains("平")){
					peakAvg.add(null);
					flatAvg.add(new BigDecimal(String.valueOf(map.get("avgValue"))).setScale(2, BigDecimal.ROUND_HALF_UP));
					valleyAvg.add(null);
				} else if (timeDivisionNmae.contains("谷")) {
					peakAvg.add(null);
					flatAvg.add(null);
					valleyAvg.add(new BigDecimal(String.valueOf(map.get("avgValue"))).setScale(2, BigDecimal.ROUND_HALF_UP));
				} else {
					peakAvg.add(null);
					flatAvg.add(null);
					valleyAvg.add(null);
				}
			}
			Map<String, Object> resultMap = new LinkedHashMap<>();
			resultMap.put("xAxisData", xAxisData);
			resultMap.put("countEnergy", countEnergy);
			resultMap.put("peakAvg", peakAvg);
			resultMap.put("flatAvg", flatAvg);
			resultMap.put("valleyAvg", valleyAvg);
			return resultMap;
		} catch (Exception e) {
			e.printStackTrace();
			logger.info("集中竞价、挂牌交易, 回演状态, 处理交易量价走势数据失败!");
			return new HashMap<>();
		}
	}

	/**
	 * map 判空
	 * @return
	 */
	public static Map<String, String> mapIsEmpty(Map<String, String> map) {
		if (map == null) {
			return new HashMap<>();
		}
		return map;
	}

	/**
	 * Map拆分(指定分组个数)
	 * @param map Map
	 * @param groupSize 分组个数 (>=1)
	 * @param <K> Key
	 * @param <V> Value
	 * @return 子Map列表
	 */
	public static <K,V> List<Map<K,V>> splitByGroupSize(Map<K,V> map, int groupSize){
		if(Objects.isNull(map) || map.isEmpty() || groupSize<1){
			//空map或者分组数<1，无法拆分
			return Collections.emptyList();
		}

		List<Map<K,V>> list = Lists.newArrayListWithCapacity(groupSize);
		if(groupSize == 1){ //只有1个分组的情况
			list.add(map);
			return list;
		}

		int mapSize = map.size(); //键值对总数
		int chunkIndex = 0; //当前分组的下标，[0, groupSize-1]
		int restCount = mapSize % groupSize; //平均后剩余的键值对数
		int chunkSize0 = mapSize / groupSize; //每个分组键值对数量
		int chunkSize1 = chunkSize0 + 1; //多分一个
		int chunkSize = chunkIndex<restCount ? chunkSize1 : chunkSize0; //实际每组的大小（前面的部分分组可能会多分1个）
		int count = 0; //每个分组的组内计数
		Map<K,V> subMap = Maps.newHashMapWithExpectedSize(chunkSize);//子Map

		for (Map.Entry<K, V> entry : map.entrySet()) {
			if(count < chunkSize){
				//每个分组按实际分组大小（chunkSize）加入键值对
				subMap.put(entry.getKey(), entry.getValue());
				count ++; //组内计数+1
			}else{
				//结束上一个分组
				list.add(subMap); //当前分组装满了->加入列表
				chunkIndex ++; //分组个数+1

				//开始下一个分组
				chunkSize = chunkIndex<restCount ? chunkSize1 : chunkSize0; //重新计算分组大小
				subMap = Maps.newHashMapWithExpectedSize(chunkSize); //新的分组
				subMap.put(entry.getKey(), entry.getValue()); //添加当前键值对
				count = 1; //组内计数重置为1
			}
		}

		list.add(subMap); //添加最后一个分组
		return list;
	}
}
