package com.alonginfo.websocket.server;


import com.alibaba.fastjson.JSONObject;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @Description websocket的实现类
 * @Author Yalon
 * @Date 2020-03-05 14:47
 * @Version mxdata_v
 */

@Component
@ServerEndpoint("/push/websocket")
@Slf4j
public class WebSocketServer {

    /**
     * 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
     */
    private static int onlineCount = 0;

    /**
     * 全部在线会话
     */
    private static Map<String, MySession> onlineSessions = new ConcurrentHashMap<>();

    /**
     * 连接建立成功调用的方法*/
    @OnOpen
    public void onOpen(Session session) {
        MySession mySession = new MySession();
        mySession.setSession(session);
        //加入set中
        onlineSessions.put(session.getId(), mySession);
        //在线数加1
        addOnlineCount();
        log.info("有新会话开始监听:{},当前在线会话为{}个", session.getId(), getOnlineCount());
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(Session session) {
        //从set中删除
        onlineSessions.remove(session.getId());
        //在线数减1
        subOnlineCount();
        log.info("{}会话连接关闭！当前在线会话为{}", session.getId(), getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     * @param session 会话
     * @param jsonStr 这里约定传递的消息为JSON字符串 方便传递更多参数！
     */
    @OnMessage
    public void onMessage(Session session, String jsonStr) {
        log.info("收到来自会话{}的消息:{}", session.getId(), jsonStr);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        String event = jsonObject.getString("event");
        JSONObject data = jsonObject.getJSONObject("data");

        //5：轮播页面发送自己pagePath/groupName
        if ("5".equals(event)) {
            Map<String,String> dataMap = onlineSessions.get(session.getId()).getData();
            dataMap.put("groupId",data.getString("groupId"));
            dataMap.put("groupName",data.getString("groupName"));
        }

        //实时页面数据传递 event = 6
        //realTimePageId: 双边协商:sb  集中竞价: jz  挂牌交易: gp  D+3: d3
        //tradeId: 交易序列id
        if ("6".equals(event)) {
            Map<String, String> dataMap = onlineSessions.get(session.getId()).getData();
            dataMap.put("realTimePageId", data.getString("realTimePageId"));
            dataMap.put("tradeId", data.getString("tradeId"));
        }
    }

    /**
     *
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("websocket发生错误");
        error.printStackTrace();
    }

    /**
     * 公共方法：发送所有在线主题分组 给user
     */
    public List<Map<String,String>> getOnlineThemeGroups() {
        List<Map<String,String>> data = new ArrayList<>();
        onlineSessions.forEach((key,mysession)->{
            if (mysession.getData().size() > 0) {
                data.add(mysession.getData());
            }
        });
        return data;
    }
 /**
     * 公共方法：发送信息给所有人
     */
    public void sendMessageToAll(String jsonMsg) {
        onlineSessions.forEach((id, session) -> {
            try {
                session.getSession().getBasicRemote().sendText(jsonMsg);
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        log.info("推送消息到全部{}个会话，推送内容:{}", getOnlineCount(), jsonMsg);
    }

    /**
     * 公共方法：发送给指定主题ID
     * @param groupId 主题分组ID
     * @param jsonMsg 推送内容
     */
    public void sendMessageToAssignUser(String groupId , String jsonMsg) {
        onlineSessions.forEach((sid, session) -> {
            try {
                if(groupId.equals(session.getData().get("groupId"))) {
                    session.getSession().getBasicRemote().sendText(jsonMsg);
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        log.info("推送消息到主题ID为{}会话，推送内容:{}", groupId, jsonMsg);
    }

    /**
     * 公共方法: 发送给指定实时页面
     * @param pageId
     * @param jsonMsg
     */
    public void pushDataToRealTimePage(String pageId, String jsonMsg) {
        onlineSessions.forEach((sid, session) -> {
            try {
                if (pageId.equals(session.getData().get("realTimePageId"))) {
                    session.getSession().getBasicRemote().sendText(jsonMsg);
                }
            }catch (Exception e) {
                e.printStackTrace();
            }
        });
        log.info("推送数据到实时页面, 页面ID: {}", pageId);
    }

    /**
     * 判断实时页面是否在线
     * @param pageId
     */
    public boolean isOnlineForRealTimePage(String pageId) {
        List<String> realTimePageId = onlineSessions.entrySet().stream().map(m -> m.getValue().getData().get("realTimePageId")).collect(Collectors.toList());
        if (realTimePageId.contains(pageId)) {
            return true;
        }
        return false;
    }

    /**
     * 通过pageId获取tradeId
     * @param pageId
     * @return
     */
    public String getTradeIdByPageId(String pageId) {
        String tradeId = onlineSessions.entrySet()
                .stream()
                .filter(m -> pageId.equals(m.getValue().getData().get("realTimePageId")))
                .map(i -> i.getValue().getData().get("tradeId")).findFirst().orElse("");
        return tradeId;
    }

    /**
     * 公共方法：发送信息给指定session
     */
    public void sendMessageBySid(String jsonMsg, String sid) {
        onlineSessions.forEach((id, session) -> {
            if (id.equals(sid)) {
                try {
                    session.getSession().getBasicRemote().sendText(jsonMsg);
                    log.info("推送消息到会话{}，推送内容:{}", sid, jsonMsg);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
    }

    private static synchronized int getOnlineCount() {
        return onlineCount;
    }

    private static synchronized void addOnlineCount() {
        WebSocketServer.onlineCount++;
    }

    private static synchronized void subOnlineCount() {
        WebSocketServer.onlineCount--;
    }

    // 存储session
    @Data
    class MySession{
        private  Session session;
        private  Map<String,String> data = new HashMap();
    }
}

